/*
 * exdate.cc
 * wxExtend date class
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#include "wx_date.h"
#include "wxstring.h"

#include "wxextend.h"

WXID wxcDateCreateString(char *s)
{
  wxDate *date = new wxDate(s);
  wxStoreThingInTable((long)date, date);
  return (WXID)date;
}

WXID wxcDateCreateJulian(long j)
{
  wxDate *date = new wxDate(j);
  wxStoreThingInTable((long)date, date);
  return (WXID)date;
}

WXID wxcDateCreate(void)
{
  wxDate *date = new wxDate;
  wxStoreThingInTable((long)date, date);
  return (WXID)date;
}

WXID wxcDateCreate(const int m, const int d, const int y)
{
  wxDate *date = new wxDate(m, d, y);
  wxStoreThingInTable((long)date, date);
  return (WXID)date;
}

Bool wxcDateDelete(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    wxDeleteThingInTable((long)date);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in date-delete: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

WXID wxcDateAddDays(WXID dateId, long days)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    wxDate *newDate = new wxDate((*date) + days);
    wxStoreThingInTable((long)newDate, newDate);
    return (WXID)newDate;
  }
  else
  {
    ExtensionError << "Error in date-add-days: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

WXID wxcDateSubtractDays(WXID dateId, long days)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    wxDate *newDate = new wxDate((*date) - days);
    wxStoreThingInTable((long)newDate, newDate);
    return (WXID)newDate;
  }
  else
  {
    ExtensionError << "Error in date-subtract-days: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

long wxcDateSubtract(WXID dateId1, WXID dateId2)
{
  wxDate *date1 = (wxDate *)wxGetTypedObject(dateId1, CLASSINFO(wxDate));
  wxDate *date2 = (wxDate *)wxGetTypedObject(dateId2, CLASSINFO(wxDate));
  if (date1 && date2)
  {
    long days = (*date1) - (*date2);
    return days;
  }
  else
  {
    ExtensionError << "Error in date-subtract: could not find one or both dates " << dateId1 << ", " << dateId2 << ".\n";
    return 0;
  }
}

Bool wxcDateAddSelf(WXID dateId, long days)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    (*date) += days;
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in date-add-self: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

Bool wxcDateSubtractSelf(WXID dateId, long days)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    (*date) -= days;
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in date-subtract-self: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

Bool wxcDateLE(WXID dateId1, WXID dateId2)
{
  wxDate *date1 = (wxDate *)wxGetTypedObject(dateId1, CLASSINFO(wxDate));
  wxDate *date2 = (wxDate *)wxGetTypedObject(dateId2, CLASSINFO(wxDate));
  if (date1 && date2)
  {
    return (*date1) < (*date2);
  }
  else
  {
    ExtensionError << "Error in date-le: could not find one or both dates " << dateId1 << ", " << dateId2 << ".\n";
    return FALSE;
  }
}

Bool wxcDateLEQ(WXID dateId1, WXID dateId2)
{
  wxDate *date1 = (wxDate *)wxGetTypedObject(dateId1, CLASSINFO(wxDate));
  wxDate *date2 = (wxDate *)wxGetTypedObject(dateId2, CLASSINFO(wxDate));
  if (date1 && date2)
  {
    return (*date1) <= (*date2);
  }
  else
  {
    ExtensionError << "Error in date-leq: could not find one or both dates " << dateId1 << ", " << dateId2 << ".\n";
    return FALSE;
  }
}

Bool wxcDateGE(WXID dateId1, WXID dateId2)
{
  wxDate *date1 = (wxDate *)wxGetTypedObject(dateId1, CLASSINFO(wxDate));
  wxDate *date2 = (wxDate *)wxGetTypedObject(dateId2, CLASSINFO(wxDate));
  if (date1 && date2)
  {
    return (*date1) > (*date2);
  }
  else
  {
    ExtensionError << "Error in date-ge: could not find one or both dates " << dateId1 << ", " << dateId2 << ".\n";
    return FALSE;
  }
}

Bool wxcDateGEQ(WXID dateId1, WXID dateId2)
{
  wxDate *date1 = (wxDate *)wxGetTypedObject(dateId1, CLASSINFO(wxDate));
  wxDate *date2 = (wxDate *)wxGetTypedObject(dateId2, CLASSINFO(wxDate));
  if (date1 && date2)
  {
    return (*date1) >= (*date2);
  }
  else
  {
    ExtensionError << "Error in date-geq: could not find one or both dates " << dateId1 << ", " << dateId2 << ".\n";
    return FALSE;
  }
}

Bool wxcDateEQ(WXID dateId1, WXID dateId2)
{
  wxDate *date1 = (wxDate *)wxGetTypedObject(dateId1, CLASSINFO(wxDate));
  wxDate *date2 = (wxDate *)wxGetTypedObject(dateId2, CLASSINFO(wxDate));
  if (date1 && date2)
  {
    return (*date1) == (*date2);
  }
  else
  {
    ExtensionError << "Error in date-eq: could not find one or both dates " << dateId1 << ", " << dateId2 << ".\n";
    return FALSE;
  }
}

Bool wxcDateNEQ(WXID dateId1, WXID dateId2)
{
  wxDate *date1 = (wxDate *)wxGetTypedObject(dateId1, CLASSINFO(wxDate));
  wxDate *date2 = (wxDate *)wxGetTypedObject(dateId2, CLASSINFO(wxDate));
  if (date1 && date2)
  {
    return (*date1) == (*date2);
  }
  else
  {
    ExtensionError << "Error in date-neq: could not find one or both dates " << dateId1 << ", " << dateId2 << ".\n";
    return FALSE;
  }
}

char *wxcDateFormat(WXID dateId, const int type)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    return date->FormatDate(type);
  }
  else
  {
    ExtensionError << "Error in date-format: could not find date " << dateId << ".\n";
    return NULL;
  }
}

Bool wxcDateSetFormat(WXID dateId, const int type)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    date->SetFormat(type);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in date-set-format: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

Bool wxcDateSetFormat(WXID dateId, const char *type)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    int format = 0;
    wxString str = type;
    if (str == "wxDAY")
      format = wxDAY;
    else if (str == "wxMONTH")
      format = wxMONTH;
    else if (str == "wxMDY")
      format = wxMDY;
    else if (str == "wxFULL")
      format = wxFULL;
    else if (str == "wxEUROPEAN")
      format = wxEUROPEAN;
    else
    {
      ExtensionError << "Error in date-set-format: unrecognised format " << (char *)type << ".\n";
      return FALSE;
    }
    date->SetFormat(format);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in date-set-format: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

Bool wxcDateSetOption(WXID dateId, const int option, Bool enable)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    date->SetOption(option, enable);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in date-set-option: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

Bool wxcDateSetOption(WXID dateId, const char *option, Bool enable)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    int format = 0;
    wxString str = option;
    if (str == "wxNO_CENTURY")
      format = wxNO_CENTURY;
    else if (str == "wxDATE_ABBR")
      format = wxDATE_ABBR;
    else
    {
      ExtensionError << "Error in date-set-option: unrecognised option " << (char *)option << ".\n";
      return FALSE;
    }
    date->SetOption(format, enable);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in date-set-option: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

long wxcDateGetJulianDate(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    return date->GetJulianDate();
  }
  else
  {
    ExtensionError << "Error in date-get-julian-date: could not find date " << dateId << ".\n";
    return 0;
  }
}

int wxcDateGetDayOfYear(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    return date->GetDayOfYear();
  }
  else
  {
    ExtensionError << "Error in date-get-day-of-year: could not find date " << dateId << ".\n";
    return 0;
  }
}

Bool wxcDateIsLeapYear(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    return date->IsLeapYear();
  }
  else
  {
    ExtensionError << "Error in date-is-leap-year: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

Bool wxcDateSetCurrentDate(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    (void) date->Set();
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in date-set-current-date: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

Bool wxcDateSetJulian(WXID dateId, long julian)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    (void) date->Set(julian);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in date-set-julian: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

Bool wxcDateSetDate(WXID dateId, int m, int d, int y)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    (void) date->Set(m, d, y);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in date-set-date: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

Bool wxcDateAddWeeks(WXID dateId, int n)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    (void) date->AddWeeks(n);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in date-add-weeks: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

Bool wxcDateAddMonths(WXID dateId, int n)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    (void) date->AddMonths(n);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in date-add-months: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

Bool wxcDateAddYears(WXID dateId, int n)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    (void) date->AddYears(n);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in date-add-years: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

int wxcDateGetDay(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    return date->GetDay();
  }
  else
  {
    ExtensionError << "Error in date-get-day: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

int wxcDateGetDaysInMonth(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    return date->GetDaysInMonth();
  }
  else
  {
    ExtensionError << "Error in date-get-days-in-month: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

int wxcDateGetFirstDayOfMonth(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    return date->GetFirstDayOfMonth();
  }
  else
  {
    ExtensionError << "Error in date-get-first-day-of-month: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

int wxcDateGetDayOfWeek(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    return date->GetDayOfWeek();
  }
  else
  {
    ExtensionError << "Error in date-get-day-of-week: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

int wxcDateGetWeekOfMonth(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    return date->GetWeekOfMonth();
  }
  else
  {
    ExtensionError << "Error in date-get-week-of-month: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

int wxcDateGetWeekOfYear(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    return date->GetWeekOfYear();
  }
  else
  {
    ExtensionError << "Error in date-get-week-of-year: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

char *wxcDateGetDayOfWeekName(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    return date->GetDayOfWeekName();
  }
  else
  {
    ExtensionError << "Error in date-get-day-of-week-name: could not find date " << dateId << ".\n";
    return NULL;
  }
}

char *wxcDateGetMonthName(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    return date->GetMonthName();
  }
  else
  {
    ExtensionError << "Error in date-get-month-name: could not find date " << dateId << ".\n";
    return NULL;
  }
}

int wxcDateGetMonth(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    return date->GetMonth();
  }
  else
  {
    ExtensionError << "Error in date-get-month: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

int wxcDateGetYear(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    return date->GetYear();
  }
  else
  {
    ExtensionError << "Error in date-get-year: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

WXID wxcDateGetMonthStart(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    wxDate *newDate = new wxDate(date->GetMonthStart());
    wxStoreThingInTable((long)newDate, newDate);
    return (WXID)newDate;
  }
  else
  {
    ExtensionError << "Error in date-get-month-start: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

WXID wxcDateGetMonthEnd(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    wxDate *newDate = new wxDate(date->GetMonthEnd());
    wxStoreThingInTable((long)newDate, newDate);
    return (WXID)newDate;
  }
  else
  {
    ExtensionError << "Error in date-get-month-end: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

WXID wxcDateGetYearStart(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    wxDate *newDate = new wxDate(date->GetYearStart());
    wxStoreThingInTable((long)newDate, newDate);
    return (WXID)newDate;
  }
  else
  {
    ExtensionError << "Error in date-get-year-start: could not find date " << dateId << ".\n";
    return FALSE;
  }
}

WXID wxcDateGetYearEnd(WXID dateId)
{
  wxDate *date = (wxDate *)wxGetTypedObject(dateId, CLASSINFO(wxDate));
  if (date)
  {
    wxDate *newDate = new wxDate(date->GetYearEnd());
    wxStoreThingInTable((long)newDate, newDate);
    return (WXID)newDate;
  }
  else
  {
    ExtensionError << "Error in date-get-year-end: could not find date " << dateId << ".\n";
    return FALSE;
  }
}


