/*
 * wxcwin.cc
 * wxWindows command processing: windows
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#include "wx_hash.h"
#include "wx_types.h"
#include "wx_res.h"
#include "wxstring.h"

#include <iostream.h>
#include <ctype.h>
#include <stdlib.h>

#include "wxextend.h"
#include "wxextenp.h"
#include "exitemsp.h"

// Record what frames and dialogs are used so we can clear
// them up if necessary
wxList wxExtensionFrameList;

wxItem *wxClipsPanelCreateItem(wxPanel *panel, wxItemResource *resource, wxResourceTable *table);

// From exmisc.cc
extern wxResourceTable wxExtensionResourceTable;

/************************************************************************
 * SPECIAL-PURPOSE WINDOWS/WIDGETS
 *
 ************************************************************************/

// Dialog box
class wxExtensionDialogBox:public wxDialogBox
{
  public:
  wxExtensionDialogBox (wxWindow * parent, char *title,
			Bool modal = FALSE,
			int x = -1, int y = -1,
			int width = -1, int height = -1, long style = 0, char *name = "dialogBox"):
    wxDialogBox (parent, title, modal, x, y, width, height, style, name)
  {
    wxStoreThingInTable ((long) this, this, CLASSINFO(wxDialogBox));
    wxExtensionFrameList.Append (this);

    wxCanvasDC *dc = GetDC ();
    if (dc)
        wxStoreThingInTable ((long) dc, dc, CLASSINFO(wxCanvasDC));
  }
  wxExtensionDialogBox (void)
  {
    wxStoreThingInTable ((long) this, this, CLASSINFO(wxDialogBox));
    wxExtensionFrameList.Append (this);
  }

   ~wxExtensionDialogBox ()
  {
    wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxDialogBox), "OnDelete");
    if (func)
      (*(void (*)(long)) func) ((long) this);

    wxDeleteThingInTable ((long) this);
    wxExtensionFrameList.DeleteObject (this);

    wxCanvasDC *dc = GetDC ();
    wxDeleteThingInTable ((long) dc);
  }
  void OnSize (int w, int h);
  Bool OnClose (void);
  void OnPaint (void);
  void OnEvent (wxMouseEvent & event);
  void OnDefaultAction (wxItem * item);
  void OnCommand (wxWindow& win, wxCommandEvent& event);

  wxItem *CreateItem(wxItemResource *childResource, wxResourceTable *table = NULL)
  {
    return wxClipsPanelCreateItem(this, childResource, table);
  }
};

void wxExtensionDialogBox::OnSize (int w, int h)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxDialogBox), "OnSize");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int, int)) func) (&handled, (long) this, w, h);

      if (!handled)
        wxDialogBox::OnSize (w, h);
    }
  else
    wxDialogBox::OnSize (w, h);
}

Bool wxExtensionDialogBox::OnClose (void)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxDialogBox), "OnClose");
  if (func)
    {
      Bool handled = FALSE;
      Bool returnValue = (*(Bool (*)(Bool *, long)) func) (&handled, (long) this);

      if (!handled)
        return wxDialogBox::OnClose ();
      else
	return returnValue;
    }
  else
    return wxDialogBox::OnClose ();
}

void wxExtensionDialogBox::OnPaint (void)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxDialogBox), "OnPaint");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long)) func) (&handled, (long) this);

      if (!handled)
        wxDialogBox::OnPaint ();
    }
  else
    wxDialogBox::OnPaint ();
}

void wxExtensionDialogBox::OnEvent (wxMouseEvent & event)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxDialogBox), "OnEvent");
  if (func)
    {
      wxStoreThingInTable ((long) &event, &event, CLASSINFO(wxMouseEvent));

      Bool handled = FALSE;

      (*(void (*)(Bool *, long, long)) func) (&handled, (long) this, (long) &event);

      wxDeleteThingInTable ((long) &event);

      if (!handled)
        wxDialogBox::OnEvent (event);
    }
  else
    wxDialogBox::OnEvent (event);
}

void wxExtensionDialogBox::OnDefaultAction (wxItem * item)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxDialogBox), "OnDefaultAction");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, long)) func) (&handled, (long) this, (long) item);

      if (!handled)
        wxDialogBox::OnDefaultAction (item);
    }
  else
    wxDialogBox::OnDefaultAction (item);
}

void wxExtensionDialogBox::OnCommand (wxWindow& win, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxDialogBox), "OnCommand");
  if (func)
    {
      wxStoreThingInTable ((long) &event, &event, CLASSINFO(wxCommandEvent));

      Bool handled = FALSE;
      (*(void (*)(Bool *, long, long, long)) func) (&handled, (long) this, (long) &win, (long) &event);

      wxDeleteThingInTable ((long) &event);

      if (!handled)
        wxDialogBox::OnCommand (win, event);
    }
  else
    wxDialogBox::OnCommand (win, event);
}

// Panel
class wxExtensionPanel:public wxPanel
{
  public:
  wxExtensionPanel (wxWindow * parent,
		    int x = -1, int y = -1,
		    int width = -1, int height = -1, long style = 0, char *name = "panel"):
    wxPanel (parent, x, y, width, height, style, name)
  {
    wxStoreThingInTable ((long) this, this, CLASSINFO(wxPanel));
    wxCanvasDC *dc = GetDC ();
    if (dc)
        wxStoreThingInTable ((long) dc, dc, CLASSINFO(wxCanvasDC));
  }
  wxExtensionPanel (void)
  {
    wxStoreThingInTable ((long) this, this, CLASSINFO(wxPanel));
  }

   ~wxExtensionPanel ()
  {
    wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxPanel), "OnDelete");
    if (func)
      (*(void (*)(long)) func) ((long) this);

    wxDeleteThingInTable ((long) this);
    wxCanvasDC *dc = GetDC ();
    wxDeleteThingInTable ((long) dc);
  }

  void OnSize (int w, int h);
  void OnPaint (void);
  void OnEvent (wxMouseEvent & event);
  void OnDefaultAction (wxItem * item);
  void OnCommand (wxWindow& win, wxCommandEvent& event);
  wxItem *CreateItem(wxItemResource *childResource, wxResourceTable *table = NULL)
  {
    return wxClipsPanelCreateItem(this, childResource, table);
  }
};

void wxExtensionPanel::OnSize (int w, int h)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxPanel), "OnSize");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int, int)) func) (&handled, (long) this, w, h);

      if (!handled)
        wxPanel::OnSize (w, h);
    }
  else
    wxPanel::OnSize (w, h);
}

void wxExtensionPanel::OnPaint (void)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxPanel), "OnPaint");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long)) func) (&handled, (long) this);

      if (!handled)
        wxPanel::OnPaint ();
    }
  else
    wxPanel::OnPaint ();
}

void wxExtensionPanel::OnEvent (wxMouseEvent & event)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxPanel), "OnEvent");
  if (func)
    {
      wxStoreThingInTable ((long) &event, &event, CLASSINFO(wxMouseEvent));

      Bool handled = FALSE;

      (*(void (*)(Bool *, long, long)) func) (&handled, (long) this, (long) &event);

      wxDeleteThingInTable ((long) &event);

      if (!handled)
        wxPanel::OnEvent (event);
    }
  else
    wxPanel::OnEvent (event);
}

void wxExtensionPanel::OnDefaultAction (wxItem * item)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxPanel), "OnDefaultAction");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, long)) func) (&handled, (long) this, (long) item);

      if (!handled)
        wxPanel::OnDefaultAction (item);
    }
  else
    wxPanel::OnDefaultAction (item);
}

void wxExtensionPanel::OnCommand (wxWindow& win, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxPanel), "OnCommand");
  if (func)
    {
      wxStoreThingInTable ((long) &event, &event, CLASSINFO(wxCommandEvent));

      Bool handled = FALSE;
      (*(void (*)(Bool *, long, long, long)) func) (&handled, (long) this, (long) &win, (long) &event);

      wxDeleteThingInTable ((long) &event);

      if (!handled)
        wxPanel::OnCommand (win, event);
    }
  else
    wxPanel::OnCommand (win, event);
}

// Canvas
class wxExtensionCanvas:public wxCanvas
{
  public:
  wxExtensionCanvas (wxWindow * parent,
		     int x = -1, int y = -1,
		     int width = -1, int height = -1, long style = 0, char *name = "canvas"):
    wxCanvas (parent, x, y, width, height, style, name)
  {
    wxStoreThingInTable ((long) this, this, CLASSINFO(wxCanvas));
    wxCanvasDC *dc = GetDC ();
      wxStoreThingInTable ((long) dc, dc, CLASSINFO(wxCanvasDC));
  }

   ~wxExtensionCanvas ()
  {
    wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxCanvas), "OnDelete");
    if (func)
      (*(void (*)(long)) func) ((long) this);

    wxDeleteThingInTable ((long) this);
    wxCanvasDC *dc = GetDC ();
    wxDeleteThingInTable ((long) dc);
  }

  void OnSize (int w, int h);

  void OnChar (wxKeyEvent & event);
  void OnEvent (wxMouseEvent & event);
  void OnScroll (wxCommandEvent & event);
  void OnPaint (void);
};

void wxExtensionCanvas::OnSize (int w, int h)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxCanvas), "OnSize");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int, int)) func) (&handled, (long) this, w, h);

      if (!handled)
        wxCanvas::OnSize (w, h);
    }
  else
    wxCanvas::OnSize (w, h);
}

void wxExtensionCanvas::OnChar (wxKeyEvent & event)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxCanvas), "OnChar");
  if (func)
    {
      wxStoreThingInTable ((long) &event, &event, CLASSINFO(wxKeyEvent));

      Bool handled = FALSE;
      (*(void (*)(Bool *, long, long, long)) func)
	(&handled, (long) this, event.KeyCode (), (long) &event);

      wxDeleteThingInTable ((long) &event);

      if (!handled)
        wxCanvas::OnChar (event);
    }
  else
    wxCanvas::OnChar (event);
}

void wxExtensionCanvas::OnEvent (wxMouseEvent & event)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxCanvas), "OnEvent");
  if (func)
    {
      wxStoreThingInTable ((long) &event, &event, CLASSINFO(wxMouseEvent));

      Bool handled = FALSE;

      (*(void (*)(Bool *, long, long)) func) (&handled, (long) this, (long) &event);

      wxDeleteThingInTable ((long) &event);

      if (!handled)
        wxCanvas::OnEvent (event);
    }
  else
    wxCanvas::OnEvent (event);
}

void wxExtensionCanvas::OnScroll (wxCommandEvent & event)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxCanvas), "OnScroll");
  if (func)
  {
      wxStoreThingInTable ((long) &event, &event, CLASSINFO(wxCommandEvent));

      Bool handled = FALSE;

      (*(void (*)(Bool *, long, long)) func) (&handled, (long) this, (long) &event);

      wxDeleteThingInTable ((long) &event);

      if (!handled)
        wxCanvas::OnScroll (event);
    }
  else
    wxCanvas::OnScroll (event);
}

void wxExtensionCanvas::OnPaint (void)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxCanvas), "OnPaint");
  if (func)
    {
      Bool handled = FALSE;

      (*(void (*)(Bool *, long)) func) (&handled, (long) this);

      if (!handled)
        wxCanvas::OnPaint ();
    }
}

// Text subwindow
class wxExtensionTextWindow:public wxTextWindow
{
  public:
  wxExtensionTextWindow (wxWindow * parent,
			 int x = -1, int y = -1,
			 int width = -1, int height = -1, long style = 0, char *name = "textWindow"):
    wxTextWindow (parent, x, y, width, height, style, name)
  {
    wxStoreThingInTable ((long) this, this, CLASSINFO(wxTextWindow));
  }

   ~wxExtensionTextWindow ()
  {
    wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxTextWindow), "OnDelete");
    if (func)
      (*(void (*)(long)) func) ((long) this);
    wxDeleteThingInTable ((long) this);
  }

  void OnSize (int w, int h);
  void OnChar (wxKeyEvent & event);
};

void wxExtensionTextWindow::OnSize (int w, int h)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxTextWindow), "OnSize");
  if (func)
    {
      Bool handled = FALSE;

      (*(void (*)(Bool *, long, int, int)) func) (&handled, (long) this, w, h);

      if (!handled)
        wxTextWindow::OnSize (w, h);
    }
  else
    wxTextWindow::OnSize (w, h);
}

void wxExtensionTextWindow::OnChar (wxKeyEvent & event)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxTextWindow), "OnChar");
  if (func)
    {
      wxStoreThingInTable ((long) &event, &event, CLASSINFO(wxKeyEvent));

      Bool handled = FALSE;
      (*(void (*)(Bool *, long, long, long)) func)
	(&handled, (long) this, event.KeyCode (), (long) &event);

      wxDeleteThingInTable ((long) &event);

      if (!handled)
        wxTextWindow::OnChar (event);
    }
  else
    wxTextWindow::OnChar (event);
}

// Frame
class wxExtensionFrame:public wxFrame
{
  public:
  wxExtensionFrame (wxFrame * parent, char *title,
		    int x = -1, int y = -1, int width = -1, int height = -1,
		    long style = wxSDI | wxDEFAULT_FRAME, char *name = "frame"):
     wxFrame (parent, title, x, y, width, height, style, name)
  {
    wxStoreThingInTable ((long) this, this, CLASSINFO(wxFrame));
    wxExtensionFrameList.Append (this);
  }

   ~wxExtensionFrame ()
  {
    wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxFrame), "OnDelete");
    if (func)
      (*(void (*)(long)) func) ((long) this);

    wxDeleteThingInTable ((long) this);
    wxExtensionFrameList.DeleteObject (this);
  }

  void OnSize (int w, int h);
  void OnMenuCommand (int id);
  void OnMenuSelect (int id);
  void OnActivate (Bool active);
  Bool OnClose (void);
  Bool OnCharHook (wxKeyEvent& event);
};

void wxExtensionFrame::OnSize (int w, int h)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxFrame), "OnSize");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int, int)) func) (&handled, (long) this, w, h);

      if (!handled)
        wxFrame::OnSize (w, h);
    }
  else
    wxFrame::OnSize (w, h);
}

void wxExtensionFrame::OnMenuCommand (int id)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxFrame), "OnMenuCommand");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int)) func) (&handled, (long) this, id);

      if (!handled)
        wxFrame::OnMenuCommand (id);
    }
  else
    wxFrame::OnMenuCommand (id);
}

void wxExtensionFrame::OnMenuSelect (int id)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxFrame), "OnMenuSelect");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int)) func) (&handled, (long) this, id);

      if (!handled)
        wxFrame::OnMenuSelect (id);
    }
  else
    wxFrame::OnMenuSelect (id);
}

void wxExtensionFrame::OnActivate (Bool active)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxFrame), "OnActivate");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, Bool)) func) (&handled, (long) this, active);

      if (!handled)
        wxFrame::OnActivate (active);
    }
  else
    wxFrame::OnActivate (active);
}

Bool wxExtensionFrame::OnClose (void)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxFrame), "OnClose");
  if (func)
    {
      Bool handled = FALSE;
      Bool returnValue = (*(Bool (*)(Bool *, long)) func) (&handled, (long) this);

      if (!handled)
        return wxFrame::OnClose ();
      else
	return returnValue;
    }
  else
    return wxFrame::OnClose ();
}

Bool wxExtensionFrame::OnCharHook (wxKeyEvent & event)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxWindow), "OnCharHook");
  if (func)
    {
      wxStoreThingInTable ((long) &event, &event, CLASSINFO(wxKeyEvent));

      Bool handled = FALSE;
      Bool res = (*(Bool (*)(Bool *, long, long)) func)
	(&handled, (long) this, (long) &event);

      wxDeleteThingInTable ((long) &event);

      if (!handled)
        return wxFrame::OnCharHook (event);
      else return res;
    }
  else
    return wxFrame::OnCharHook (event);
}

/************************************************************************
 * GENERIC WINDOW FUNCTIONS
 *
 ************************************************************************/

// Delete window
Bool wxcWindowDelete (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      delete win;
      wxDeleteThingInTable (id);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-delete: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Show window
Bool wxcWindowShow (WXID id, Bool show)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      win->Show (show);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-show: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Is window shown?
Bool wxcWindowIsShown (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      return win->IsShown ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-is-shown: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Delete window
long wxcWindowGetNextChild (WXID id, WXID childId)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      wxNode *nextNode = NULL;
      if (childId == 0)
        nextNode = win->GetChildren()->First();
      else
      {
        wxWindow *child = (wxWindow *) wxGetTypedObject (childId, CLASSINFO(wxWindow));
        wxNode *node = win->GetChildren()->Member(child);
        if (!node)
          return -1;
        nextNode = node->Next();
      }

      if (nextNode)
        return (long)nextNode->Data();
      else
        return -1;
    }
  else
    {
      ExtensionError << "Error in window-get-next-child: could not find window " << id << ".\n";
      return -1;
    }
}

// Close window with delayed deletion
long wxcWindowClose (WXID id, long force)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      return (long)win->Close((Bool)force);
    }
  else
    {
      ExtensionError << "Error in window-close: could not find window " << id << ".\n";
      return 0;
    }
}

// Set size hints
long wxcWindowSetSizeHints (WXID id, int minW, int minH, int maxW, int maxH, int incW, int incH)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      win->SetSizeHints(minW, minH, maxW, maxH, incW, incH);
      return 1;
    }
  else
    {
      ExtensionError << "Error in window-set-size-hints: could not find window " << id << ".\n";
      return 0;
    }
}

// Refresh window
long wxcWindowRefresh (WXID id, Bool eraseBackground, int x, int y, int w, int h)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      wxRectangle rect;
      if (x != -1 && y != -1 && w != -1 && h != 1)
      {
        rect.x = x; rect.y = y; rect.width = 1; rect.height = h;
        win->Refresh(eraseBackground, &rect);
      }
      else
        win->Refresh(eraseBackground);
        
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-refresh: could not find window " << id << ".\n";
      return 0;
    }
}

// Associate a menu to a window as a Popup 
Bool wxcWindowPopupMenu (WXID id, WXID menuId, float x, float y)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      wxMenu *menu = (wxMenu *) wxGetTypedObject (menuId, CLASSINFO(wxMenu));
      if (menu)
	{
	  return win->PopupMenu (menu, x, y) ? TRUE : FALSE;
	}
      ExtensionError << "\
Error in window-popup-menu: could not find " << "menu " << menuId << ".\n";
    }
  else
    ExtensionError << "\
Error in window-popup-menu: could not find " << "window " << id << ".\n";
  return FALSE;			// Error

}


// Get window parent
WXID wxcWindowGetParent (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      wxWindow *parent = win->GetParent ();
      if (parent)
	return (long) parent;
      else
	return FALSE;
    }
  else
    {
      ExtensionError << "Error in window-get-parent: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Get window width
int wxcWindowGetWidth (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      int w, h;
      win->GetSize (&w, &h);
      return w;
    }
  else
    {
      ExtensionError << "Error in window-get-width: could not find window " << id << ".\n";
      return 0;
    }
}

// Get window height
int wxcWindowGetHeight (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      int w, h;
      win->GetSize (&w, &h);
      return h;
    }
  else
    {
      ExtensionError << "Error in window-get-height: could not find window " << id << ".\n";
      return 0;
    }
}

// Get window client width
int wxcWindowGetClientWidth (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      int w, h;
      win->GetClientSize (&w, &h);
      return w;
    }
  else
    {
      ExtensionError << "Error in window-get-client-width: could not find window " << id << ".\n";
      return 0;
    }
}

// Get window client height
int wxcWindowGetClientHeight (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      int w, h;
      win->GetClientSize (&w, &h);
      return h;
    }
  else
    {
      ExtensionError << "Error in window-get-client-height: could not find window " << id << ".\n";
      return 0;
    }
}

// Get window x position
int wxcWindowGetX (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      int x, y;
      win->GetPosition (&x, &y);
      return x;
    }
  else
    {
      ExtensionError << "Error in window-get-x: could not find window " << id << ".\n";
      return 0;
    }
}

// Get window y position
int wxcWindowGetY (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      int x, y;
      win->GetPosition (&x, &y);
      return y;
    }
  else
    {
      ExtensionError << "Error in window-get-y: could not find window " << id << ".\n";
      return 0;
    }
}

// Set size
Bool wxcWindowSetSize (WXID id, int x, int y, int w, int h)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      win->SetSize (x, y, w, h);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-set-size: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Set client size
int wxcWindowSetClientSize (WXID id, int w, int h)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      win->SetClientSize (w, h);
      return 1;
    }
  else
    {
      ExtensionError << "Error in window-set-client-size: could not find window " << id << ".\n";
      return 0;
    }
}

// Set focus
Bool wxcWindowSetFocus (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      win->SetFocus ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-set-focus: could not find window " << id << ".\n";
      return FALSE;
    }
}

Bool wxcWindowSetCursor (WXID id, WXID cursorId)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      wxCursor *cursor = (wxCursor *) wxGetTypedObject (cursorId, CLASSINFO(wxCursor));
      if (cursor)
	{
	  win->SetCursor (cursor);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in window-set-cursor: could not find cursor " << cursorId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in win-set-cursor: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Fit
Bool wxcWindowFit (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      win->Fit ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-fit: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Centre (string version)
Bool wxcWindowCentre (WXID id, char *stringDir)
{
  int direction = wxBOTH;
  if (stringDir)
    {
      if (strcmp (stringDir, "wxBOTH") == 0)
	direction = wxBOTH;
      else if (strcmp (stringDir, "wxHORIZONTAL") == 0)
	direction = wxHORIZONTAL;
      else if (strcmp (stringDir, "wxVERTICAL") == 0)
	direction = wxVERTICAL;
      else
	{
	  ExtensionError << "Error in window-centre: unrecognised direction " << direction << "\n.";
	  ExtensionError << "Should be wxHORIZONTAL, wxVERTICAL or wxBOTH.\n";
	  return FALSE;
	}
    }

  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      win->Centre (direction);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-centre: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Centre
Bool wxcWindowCentre (WXID id, int direction)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      win->Centre (direction);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-centre: could not find window " << id << ".\n";
      return FALSE;
    }
}

// Make this window modal (i.e. disable all other windows)
// Experimental.
Bool wxcWindowMakeModal (WXID id, Bool modal)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      win->MakeModal (modal);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in window-modal: could not find window " << id << ".\n";
      return FALSE;
    }
}

long wxcWindowGetSize (WXID id, int *w, int *h)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      win->GetSize (w, h);
      return 1;
    }
  else
    {
      ExtensionError << "Error in window-get-size: could not find window "
	<< id << ".\n";
      return 0;
    }
}



long wxcWindowGetClientSize (WXID id, int *w, int *h)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      win->GetClientSize (w, h);
      return 1;
    }
  else
    {
      ExtensionError << "Error in window-get-client-size: could not find window "
	<< id << ".\n";
      return 0;
    }
}

long wxcWindowGetPosition (WXID id, int *x, int *y)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      win->GetPosition (x, y);
      return 1;
    }
  else
    {
      ExtensionError << "Error in window-get-x: could not find window "
	<< id << ".\n";
      return 0;
    }
}

float wxcWindowGetCharWidth (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      return win->GetCharWidth ();
    }
  else
    {
      ExtensionError << "Error in window-get-char-width: could not find window "
	<< id << ".\n";
      return (float)0.0;
    }
}

float wxcWindowGetCharHeight (WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      return win->GetCharHeight ();
    }
  else
    {
      ExtensionError << "Error in window-get-char-height: could not find window "
	<< id << ".\n";
      return (float)0.0;
    }
}

long wxcWindowGetTextExtent (WXID id, char *text, float *width, float *height)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      win->GetTextExtent (text, width, height);
      return 1;
    }
  else
    {
      ExtensionError << "Error in window-get-text-extent: could not find window "
	<< id << ".\n";
      return 0;
    }
}

char *wxcWindowGetName(WXID id)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      return win->GetName();
    }
  else
    {
      ExtensionError << "Error in window-get-name: could not find window "
	<< id << ".\n";
      return NULL;
    }
}

long wxcWindowEnable (WXID id, Bool enable)
{
  wxWindow *win = (wxWindow *) wxGetTypedObject (id, CLASSINFO(wxWindow));
  if (win)
    {
      win->Enable (enable);
      return 1;
    }
  else
    {
      ExtensionError << "Error in window-enable: could not find window " << id << ".\n";
      return 0;
    }
}

// Get active window: but only one which has been registered with wxExtend
WXID wxcGetActiveWindow (void)
{
#ifdef wx_msw
  wxWindow *win = wxGetActiveWindow();
  if (!win)
    return -1;
    
  if (!wxGetThingFromTable ((WXID)win))
    return -1;
    
  return (WXID)win;
#else
   ExtensionError << "Error in get-active-window: this function only works under Windows.\n";
   return -1;
#endif
}

/************************************************************************
 * DIALOG AND PANEL CREATION
 *
 ************************************************************************/

// Create a new dialog box
WXID wxcDialogBoxCreate (WXID parentId, char *title, Bool modal,
		    int x, int y, int width, int height, long style, char *name)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, CLASSINFO(wxWindow));
      if (!parent)
	{
	  ExtensionError << "Error in dialog-box-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }

  wxExtensionDialogBox *dialog_box = new wxExtensionDialogBox (parent, title, modal,
						x, y, width, height, style, name);
  return (long) dialog_box;
}

// Create a new dialog box (string version)
WXID wxcDialogBoxCreate (WXID parentId, char *title, Bool modal,
		    int x, int y, int width, int height, char *styleString, char *name)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, CLASSINFO(wxWindow));
      if (!parent)
	{
	  ExtensionError << "Error in dialog-box-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }
  long style = wxExtensionParseBitList (styleString);

  wxExtensionDialogBox *dialog_box = new wxExtensionDialogBox (parent, title, modal,
						x, y, width, height, style, name);
  return (long) dialog_box;
}

WXID wxcDialogBoxCreateFromResource(WXID parentId, char *resourceName)
{
  wxWindow *win = NULL;

  if (parentId != 0)
    {
      win = (wxWindow *) wxGetTypedObject (parentId, CLASSINFO(wxWindow));
      if (!win)
	{
	  ExtensionError << "Error in dialog-box-create-from-resource: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }

  wxExtensionDialogBox *dialog = new wxExtensionDialogBox;
  if (!dialog->LoadFromResource(win, resourceName, &wxExtensionResourceTable))
  {
    delete dialog;
    return -1;
  }
  wxCanvasDC *dc = dialog->GetDC ();
  if (dc)
    wxStoreThingInTable ((long) dc, dc, CLASSINFO(wxCanvasDC));

  return (long) dialog;
}

Bool wxcDialogBoxIsModal(WXID id)
{
  wxDialogBox *dialog = (wxDialogBox *) wxGetTypedObject (id, CLASSINFO(wxDialogBox));
  if (dialog)
    {
      return dialog->IsModal();
    }
  else
    {
      ExtensionError << "Error in dialog-box-is-modal: could not find dialog box " << id << ".\n";
      return FALSE;
    }
}

void wxcDialogBoxSetModal(WXID id, Bool flag)
{
  wxDialogBox *dialog = (wxDialogBox *) wxGetTypedObject (id, CLASSINFO(wxDialogBox));
  if (dialog)
    {
      dialog->SetModal(flag);
    }
  else
    {
      ExtensionError << "Error in dialog-box-set-modal: could not find dialog box " << id << ".\n";
    }
}

// Create a new panel
long wxcPanelCreate (long parentId, int x, int y, int width, int height, long style, char *name)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, CLASSINFO(wxWindow));
      if (!parent)
	{
	  ExtensionError << "Error in panel-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }

  wxExtensionPanel *panel = new wxExtensionPanel (parent, (int) x, (int) y, (int) width, (int) height, style, name);
  return (long) panel;
}

// Panel creation (string version)
long wxcPanelCreate (long parentId, int x, int y, int width, int height, char *styleString, char *name)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, CLASSINFO(wxWindow));
      if (!parent)
	{
	  ExtensionError << "Error in panel-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }
  long style = wxExtensionParseBitList (styleString);

  wxExtensionPanel *panel = new wxExtensionPanel (parent, (int) x, (int) y, (int) width, (int) height, style, name);
  return (long) panel;
}

WXID wxcPanelCreateFromResource(WXID parentId, char *resourceName)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, CLASSINFO(wxWindow));
      if (!parent)
	{
	  ExtensionError << "Error in panel-create-from-resource: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }

  wxExtensionPanel *panel = new wxExtensionPanel;
  if (!panel->LoadFromResource(parent, resourceName, &wxExtensionResourceTable))
  {
    delete panel;
    return -1;
  }
  wxCanvasDC *dc = panel->GetDC ();
  if (dc)
    wxStoreThingInTable ((long) dc, dc, CLASSINFO(wxCanvasDC));

  return (long) panel;
}

// Set the label position
Bool wxcPanelSetLabelPosition (WXID id, int position)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, CLASSINFO(wxPanel));
  if (panel)
    {
      panel->SetLabelPosition (position);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in panel-set-label-position: could not find panel " << id << ".\n";
      return FALSE;
    }
}

// Set the label position (string version)
Bool wxcPanelSetLabelPosition (WXID id, char *position)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, CLASSINFO(wxPanel));
  if (panel)
    {
      int pos;
      if (strcmp (position, "wxVERTICAL") == 0)
	pos = wxVERTICAL;
      else if (strcmp (position, "wxHORIZONTAL") == 0)
	pos = wxHORIZONTAL;
      else
	{
	  ExtensionError << "Error in panel-set-label-position: argument should be wxVERTICAL or wxHORIZONTAL.\n";
	  return 0;
	}
      panel->SetLabelPosition (pos);
      return 1;
    }
  else
    {
      ExtensionError << "Error in panel-set-label-position: could not find panel " << id << ".\n";
      return 0;
    }
}

// Set the panel label font
Bool wxcPanelSetLabelFont (WXID id, WXID fontId)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, CLASSINFO(wxPanel));
  if (panel)
    {
      wxFont *font = (wxFont *) wxGetTypedObject (fontId, CLASSINFO(wxFont));
      if (font)
	{
	  panel->SetLabelFont (font);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in panel-set-label-font: could not find font " << fontId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in panel-set-label-font: could not find panel " << id << ".\n";
      return FALSE;
    }
}

// Set the panel button font
Bool wxcPanelSetButtonFont (WXID id, WXID fontId)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, CLASSINFO(wxPanel));
  if (panel)
    {
      wxFont *font = (wxFont *) wxGetTypedObject (fontId, CLASSINFO(wxFont));
      if (font)
	{
	  panel->SetButtonFont (font);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in panel-set-button-font: could not find font " << fontId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in panel-set-button-font: could not find panel " << id << ".\n";
      return FALSE;
    }
}

// New line
Bool wxcPanelNewLine (WXID id)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, CLASSINFO(wxPanel));
  if (panel)
    {
      panel->NewLine ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in panel-new-line: not find panel " << id << ".\n";
      return FALSE;
    }
}

// Default panel or dialog box OnPaint
Bool wxcPanelOnPaint (WXID id)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, CLASSINFO(wxPanel));
  if (panel)
    {
      panel->wxPanel::OnPaint ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in panel-on-paint: not find panel " << id << ".\n";
      return FALSE;
    }
}

// Default panel or dialog box OnEvent
Bool wxcPanelOnEvent (WXID id, WXID eventId)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, CLASSINFO(wxPanel));
  if (panel)
    {
      wxMouseEvent *event = (wxMouseEvent *) wxGetTypedObject (eventId, CLASSINFO(wxMouseEvent));
      if (event)
	{
	  panel->wxPanel::OnEvent (*event);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in panel-on-event: not find mouse event " << eventId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in panel-on-event: not find panel " << id << ".\n";
      return FALSE;
    }
}

long wxcPanelGetHorizontalSpacing (WXID id)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, CLASSINFO(wxPanel));
  if (panel)
    {
      return panel->GetHorizontalSpacing ();
    }
  else
    {
      ExtensionError << "Error in panel-get-horizontal-spacing: could not find panel " << id << ".\n";
      return FALSE;
    }
}


long wxcPanelGetVerticalSpacing (WXID id)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, CLASSINFO(wxPanel));
  if (panel)
    {
      return panel->GetVerticalSpacing ();
    }
  else
    {
      ExtensionError << "Error in panel-get-vertical-spacing: could not find panel " << id << ".\n";
      return FALSE;
    }
}

Bool wxcPanelSetHorizontalSpacing (WXID id, long spacing)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, CLASSINFO(wxPanel));
  if (panel)
    {
      panel->SetHorizontalSpacing ((int)spacing);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in panel-set-horizontal-spacing: could not find panel " << id << ".\n";
      return FALSE;
    }
}

Bool wxcPanelSetVerticalSpacing (WXID id, long spacing)
{
  wxPanel *panel = (wxPanel *) wxGetTypedObject (id, CLASSINFO(wxPanel));
  if (panel)
    {
      panel->SetVerticalSpacing ((int)spacing);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in panel-set-vertical-spacing: could not find panel " << id << ".\n";
      return FALSE;
    }
}

// Create a new canvas
WXID wxcCanvasCreate (long parentId, int x, int y, int width, int height, long style, char *name)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, CLASSINFO(wxWindow));
      if (!parent)
	{
	  ExtensionError << "Error in canvas-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }

  wxExtensionCanvas *canvas = new wxExtensionCanvas (parent, x, y, width, height, style, name);
  return (long) canvas;
}

// Create a new canvas (string version)
WXID wxcCanvasCreate (long parentId, int x, int y, int width, int height, char *styleString, char *name)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, CLASSINFO(wxWindow));
      if (!parent)
	{
	  ExtensionError << "Error in canvas-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }
  long style = wxExtensionParseBitList (styleString);

  wxExtensionCanvas *canvas = new wxExtensionCanvas (parent, x, y, width, height, style, name);
  return (long) canvas;
}

// Set the canvas scrollbars
Bool wxcCanvasSetScrollbars (WXID id, int horiz, int vert,
			int x_length, int y_length, int x_page, int y_page,
			int x_pos, int y_pos)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      canvas->SetScrollbars (horiz, vert,
			     x_length, y_length,
			     x_page, y_page,
			     x_pos, y_pos);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in canvas-set-scrollbars: could not find canvas " << id << ".\n";
      return FALSE;
    }
}

// Set the canvas scrollbars
Bool wxcCanvasScroll (WXID id, int x_pos, int y_pos)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      canvas->Scroll (x_pos, y_pos);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in canvas-scroll: could not find canvas " << id << ".\n";
      return FALSE;
    }
}

// Get canvas's device context
WXID wxcCanvasGetDC (WXID id)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      return (long) canvas->GetDC ();
    }
  else
    {
      ExtensionError << "Error in canvas-get-dc: could not find canvas " << id << ".\n";
      return 0;
    }
}

// Default canvas OnEvent
Bool wxcCanvasOnEvent (WXID id, WXID eventId)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      wxMouseEvent *event = (wxMouseEvent *) wxGetTypedObject (eventId, CLASSINFO(wxMouseEvent));
      if (event)
	{
	  canvas->wxCanvas::OnEvent (*event);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in canvas-on-event: not find mouse event " << eventId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in canvas-on-event: not find canvas " << id << ".\n";
      return FALSE;
    }
}

// Default canvas OnScroll
Bool wxcCanvasOnScroll (WXID id, WXID eventId)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      wxCommandEvent *event = (wxCommandEvent *) wxGetTypedObject (eventId, CLASSINFO(wxCommandEvent));
      if (event)
	{
	  canvas->wxCanvas::OnScroll (*event);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in canvas-on-scroll: not find command event " << eventId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in canvas-on-scroll: not find canvas " << id << ".\n";
      return FALSE;
    }
}

// Default canvas OnChar
Bool wxcCanvasOnChar (WXID id, WXID eventId)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      wxKeyEvent *event = (wxKeyEvent *) wxGetTypedObject (eventId, CLASSINFO(wxKeyEvent));
      if (event)
	{
	  canvas->wxCanvas::OnChar (*event);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in canvas-on-char: not find key event " << eventId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in canvas-on-char: not find canvas " << id << ".\n";
      return FALSE;
    }
}


int wxcCanvasViewStartX (WXID id)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      int x, y;
      canvas->ViewStart(&x, &y);
      return x;
    }
  else
    {
      ExtensionError << "Error in canvas-view-start-x: could not find canvas " << id << ".\n";
      return 0;
    }
}

int wxcCanvasViewStartY (WXID id)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      int x, y;
      canvas->ViewStart(&x, &y);
      return y;
    }
  else
    {
      ExtensionError << "Error in canvas-view-start-y: could not find canvas " << id << ".\n";
      return 0;
    }
}

void wxcCanvasSetScrollPageX (WXID id, int val)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      canvas->SetScrollPage(wxHORIZONTAL, val);
    }
  else
    {
      ExtensionError << "Error in canvas-set-scroll-page-x: could not find canvas " << id << ".\n";
      return;
    }
}

void wxcCanvasSetScrollPageY (WXID id, int val)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      canvas->SetScrollPage(wxVERTICAL, val);
    }
  else
    {
      ExtensionError << "Error in canvas-set-scroll-page-y: could not find canvas " << id << ".\n";
      return;
    }
}

void wxcCanvasSetScrollPosX (WXID id, int val)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      canvas->SetScrollPos(wxHORIZONTAL, val);
    }
  else
    {
      ExtensionError << "Error in canvas-set-scroll-pos-x: could not find canvas " << id << ".\n";
      return;
    }
}

void wxcCanvasSetScrollPosY (WXID id, int val)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      canvas->SetScrollPos(wxVERTICAL, val);
    }
  else
    {
      ExtensionError << "Error in canvas-set-scroll-pos-y: could not find canvas " << id << ".\n";
      return;
    }
}

void wxcCanvasSetScrollRangeX (WXID id, int val)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      canvas->SetScrollRange(wxHORIZONTAL, val);
    }
  else
    {
      ExtensionError << "Error in canvas-set-scroll-range-x: could not find canvas " << id << ".\n";
      return;
    }
}

void wxcCanvasSetScrollRangeY (WXID id, int val)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      canvas->SetScrollRange(wxVERTICAL, val);
    }
  else
    {
      ExtensionError << "Error in canvas-set-scroll-range-y: could not find canvas " << id << ".\n";
      return;
    }
}

int wxcCanvasGetScrollPageX (WXID id)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      return canvas->GetScrollPage(wxHORIZONTAL);
    }
  else
    {
      ExtensionError << "Error in canvas-get-scroll-page-x: could not find canvas " << id << ".\n";
      return 0;
    }
}

int wxcCanvasGetScrollPageY (WXID id)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      return canvas->GetScrollPage(wxVERTICAL);
    }
  else
    {
      ExtensionError << "Error in canvas-get-scroll-page-y: could not find canvas " << id << ".\n";
      return 0;
    }
}

int wxcCanvasGetScrollPosX (WXID id)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      return canvas->GetScrollPos(wxHORIZONTAL);
    }
  else
    {
      ExtensionError << "Error in canvas-get-scroll-pos-x: could not find canvas " << id << ".\n";
      return 0;
    }
}

int wxcCanvasGetScrollPosY (WXID id)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      return canvas->GetScrollPos(wxVERTICAL);
    }
  else
    {
      ExtensionError << "Error in canvas-get-scroll-pos-y: could not find canvas " << id << ".\n";
      return 0;
    }
}

int wxcCanvasGetScrollRangeX (WXID id)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      return canvas->GetScrollRange(wxHORIZONTAL);
    }
  else
    {
      ExtensionError << "Error in canvas-get-scroll-range-x: could not find canvas " << id << ".\n";
      return 0;
    }
}

int wxcCanvasGetScrollRangeY (WXID id)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      return canvas->GetScrollPos(wxVERTICAL);
    }
  else
    {
      ExtensionError << "Error in canvas-get-scroll-range-y: could not find canvas " << id << ".\n";
      return 0;
    }
}

int wxcCanvasGetScrollPixelsPerUnitX (WXID id)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      int x, y;
      canvas->GetScrollPixelsPerUnit(&x, &y);
      return x;
    }
  else
    {
      ExtensionError << "Error in canvas-get-scroll-pixels-per-unit-x: could not find canvas " << id << ".\n";
      return 0;
    }
}

int wxcCanvasGetScrollPixelsPerUnitY (WXID id)
{
  wxCanvas *canvas = (wxCanvas *) wxGetTypedObject (id, CLASSINFO(wxCanvas));
  if (canvas)
    {
      int x, y;
      canvas->GetScrollPixelsPerUnit(&x, &y);
      return y;
    }
  else
    {
      ExtensionError << "Error in canvas-get-scroll-pixels-per-unit-y: could not find canvas " << id << ".\n";
      return 0;
    }
}







// Create a new text window
WXID wxcTextWindowCreate (WXID parentId, int x, int y, int width, int height,
		     long style, char *name)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, CLASSINFO(wxWindow));
      if (!parent)
	{
	  ExtensionError << "Error in text-window-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }
  wxExtensionTextWindow *text_window =
  new wxExtensionTextWindow (parent, x, y, width, height, style, name);
  return (long) text_window;
}

// Create a new text window (string version)
WXID wxcTextWindowCreate (long parentId, int x, int y, int width, int height,
		     char *styleString, char *name)
{
  wxWindow *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxWindow *) wxGetTypedObject (parentId, CLASSINFO(wxWindow));
      if (!parent)
	{
	  ExtensionError << "Error in text-window-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }
  long style = wxExtensionParseBitList (styleString);

  wxExtensionTextWindow *text_window =
  new wxExtensionTextWindow (parent, x, y, width, height, style, name);
  return (long) text_window;
}

// Load a file
Bool wxcTextWindowLoadFile (WXID id, char *filename)
{
  if (!filename)
    return FALSE;

  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      return win->LoadFile (filename);
    }
  else
    {
      ExtensionError << "Error in text-window-load-file: could not find text window " << id << ".\n";
      return FALSE;
    }
}

// Save a file
Bool wxcTextWindowSaveFile (WXID id, char *filename)
{
  if (!filename)
    return FALSE;

  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      return win->SaveFile (filename);
    }
  else
    {
      ExtensionError << "Error in text-window-save-file: could not find text window " << id << ".\n";
      return FALSE;
    }
}

// Write text to the window
Bool wxcTextWindowWrite (WXID id, long val)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    *win << val;
  else
    {
      ExtensionError << "Error in text-window-write: could not find text window " << id << ".\n";
      return FALSE;
    }
  return TRUE;
}

Bool wxcTextWindowWrite (WXID id, char *val)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    *win << val;
  else
    {
      ExtensionError << "Error in text-window-write: could not find text window " << id << ".\n";
      return FALSE;
    }
  return TRUE;
}

Bool wxcTextWindowWrite (WXID id, double val)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    *win << val;
  else
    {
      ExtensionError << "Error in text-window-write: could not find text window " << id << ".\n";
      return FALSE;
    }
  return TRUE;
}

// Return TRUE if modified
Bool wxcTextWindowModified (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      return win->Modified ();
    }
  else
    {
      ExtensionError << "Error in text-window-modified: could not find text window " << id << ".\n";
      return FALSE;
    }
}

Bool wxcTextWindowClear (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      win->Clear ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in text-window-clear: could not find text window " << id << ".\n";
      return FALSE;
    }
}

// Return TRUE if modified
Bool wxcTextWindowDiscardEdits (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      win->DiscardEdits ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in text-window-discard-edits: could not find text window " << id << ".\n";
      return FALSE;
    }
}

// Copy selection to clipboard
Bool wxcTextWindowCopy (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      win->Copy ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in text-window-copy: could not find text window " << id << ".\n";
      return FALSE;
    }
}

// Paste selection from clipboard
Bool wxcTextWindowPaste (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      win->Paste ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in text-window-paste: could not find text window " << id << ".\n";
      return FALSE;
    }
}

// Copy selection to clipboard, and delete.
Bool wxcTextWindowCut (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      win->Cut ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in text-window-cut: could not find text window " << id << ".\n";
      return FALSE;
    }
}

char *wxcTextWindowGetContents (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      return win->GetContents ();
    }
  else
    {
      ExtensionError << "Error in text-window-get-contents: could not find window "
	<< id << ".\n";
      return 0;
    }
}

Bool wxcTextWindowReplace (WXID id, long start, long end, char *text)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      win->Replace (start, end, text);
      return TRUE;
    }
  ExtensionError << "\
Error in text-window-replace: could not find window " << id << ".\n";
  return FALSE;
}

Bool wxcTextWindowRemove (WXID id, long start, long end)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      win->Remove (start, end);
      return TRUE;
    }
  ExtensionError << "\
Error in text-window-remove: could not find window " << id << ".\n";
  return FALSE;
}


Bool wxcTextWindowSetEditable (WXID id, Bool editable)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      win->SetEditable (editable);
      return TRUE;
    }
  ExtensionError << "\
Error in text-window-set-editable: could not find window " << id << ".\n";
  return FALSE;
}


Bool wxcTextWindowSetFont (WXID id, WXID fontId)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      wxFont *font = (wxFont *) wxGetTypedObject (fontId, CLASSINFO(wxFont));
      if (font)
	{
	  win->SetFont (font);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "\
Error in text-window-set-font: could not find font " << fontId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "\
Error in text-window-set-font: could not find dc " << id << ".\n";
      return FALSE;
    }
}

Bool wxcTextWindowSetSelection (WXID id, long start, long end)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      win->SetSelection (start, end);
      return TRUE;
    }
  ExtensionError << "\
Error in text-window-set-selection: could not find window " << id << ".\n";
  return FALSE;
}

long wxcTextWindowGetLastPosition (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    return win->GetLastPosition ();
  ExtensionError << "\
Error in text-window-get-last-position: could not find window " << id << ".\n";
  return -1;
}

Bool wxcTextWindowShowPosition (WXID id, long position)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      win->ShowPosition (position);
      return TRUE;
    }
  ExtensionError << "Error in text-window-show-position: could not find window " << id << ".\n";
  return FALSE;
}


long wxcTextWindowXYToPosition (WXID id, long x, long y)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    return win->XYToPosition (x, y);
  ExtensionError << "\
Error in text-window-XY-to-position: could not find window " << id << ".\n";
  return -1;
}

Bool wxcTextWindowPositionToXY (WXID id, long position, long *x, long *y)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      win->PositionToXY (position, x, y);
      return TRUE;
    }
  ExtensionError << "\
Error in text-window-position-to-XY: could not find window " << id << ".\n";
  return FALSE;
}

/*
 * New stuff, 13th November 1995 JACS
 */
 
long wxcTextWindowPositionToLine (WXID id, long position)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      long x, y;
      win->PositionToXY (position, &x, &y);
      return y;
    }
  ExtensionError << "Error in text-window-position-to-line: could not find window " << id << ".\n";
  return 0;
}

long wxcTextWindowPositionToChar (WXID id, long position)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      long x, y;
      win->PositionToXY (position, &x, &y);
      return x;
    }
  ExtensionError << "Error in text-window-position-to-char: could not find window " << id << ".\n";
  return 0;
}

Bool wxcTextWindowSetInsertionPoint (WXID id, long position)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      win->SetInsertionPoint (position);
      return TRUE;
    }
  ExtensionError << "Error in text-window-set-insertion-point: could not find window " << id << ".\n";
  return FALSE;
}

long wxcTextWindowGetInsertionPoint (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      return win->GetInsertionPoint ();
      return TRUE;
    }
  ExtensionError << "Error in text-window-get-insertion-point: could not find window " << id << ".\n";
  return 0;
}

long wxcTextWindowGetLineLength (WXID id, long lineNo)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      return win->GetLineLength (lineNo);
    }
  ExtensionError << "Error in text-window-get-line-length: could not find window " << id << ".\n";
  return 0;
}

char *wxcTextWindowGetLineText (WXID id, long lineNo)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      static char buf[256];
      win->GetLineText (lineNo, buf);
      return buf;
    }
  ExtensionError << "Error in text-window-get-line-text: could not find window " << id << ".\n";
  return NULL;
}

long wxcTextWindowGetNumberOfLines (WXID id)
{
  wxTextWindow *win = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (win)
    {
      return win->GetNumberOfLines ();
    }
  ExtensionError << "Error in text-window-get-number-of-lines: could not find window " << id << ".\n";
  return 0;
}

// Default text window OnChar
Bool wxcTextWindowOnChar (WXID id, WXID eventId)
{
  wxTextWindow *textw = (wxTextWindow *) wxGetTypedObject (id, CLASSINFO(wxTextWindow));
  if (textw)
    {
      wxKeyEvent *event = (wxKeyEvent *) wxGetTypedObject (eventId, CLASSINFO(wxKeyEvent));
      if (event)
	{
	  textw->wxTextWindow::OnChar (*event);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in text-window-on-char: not find key event " << eventId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in text-window-on-char: not find panel " << id << ".\n";
      return FALSE;
    }
}

/************************************************************************
 * FRAME FUNCTIONS
 *
 ************************************************************************/

// Create a new frame
WXID wxcFrameCreate (WXID parentId, char *title, int x, int y,
		int width, int height, long style, char *name)
{
  wxFrame *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxFrame *) wxGetTypedObject (parentId, CLASSINFO(wxFrame));
      if (!parent)
	{
	  ExtensionError << "Error in frame-create: could not find parent frame " << parentId << ".\n";
	  return 0;
	}
    }

  wxExtensionFrame *new_frame = new wxExtensionFrame (parent, title, x, y,
						      width, height, style, name);
  return (long) new_frame;
}

// Create a new frame (string version)
WXID wxcFrameCreate (WXID parentId, char *title, int x, int y,
		int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList (styleString);
  wxFrame *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxFrame *) wxGetTypedObject (parentId, CLASSINFO(wxFrame));
      if (!parent)
	{
	  ExtensionError << "Error in frame-create: could not find parent frame " << parentId << ".\n";
	  return 0;
	}
    }

  wxExtensionFrame *new_frame = new wxExtensionFrame (parent, title, x, y,
						      width, height, style, name);
  return (long) new_frame;
}

// Set the menu bar
Bool wxcFrameSetMenuBar (WXID id, WXID menubar_id)
{
  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, CLASSINFO(wxFrame));
  if (frame)
    {
      wxMenuBar *menu_bar = (wxMenuBar *) wxGetTypedObject (menubar_id, CLASSINFO(wxMenuBar));
      if (menu_bar)
	{
	  frame->SetMenuBar (menu_bar);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in frame-set-menu-bar: could not find menu bar " << menubar_id << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in frame-set-menu-bar: could not find frame " << id << ".\n";
      return FALSE;
    }
}

// Set the toolbar for MDI operation
Bool wxcFrameSetToolBar (WXID id, WXID toolbar_id)
{
  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, CLASSINFO(wxFrame));
  if (frame)
    {
      wxWindow *toolbar = (wxWindow *) wxGetTypedObject (toolbar_id, CLASSINFO(wxWindow));
      if (toolbar)
	{
	  frame->SetToolBar (toolbar);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in frame-set-tool-bar: could not find toolbar " << toolbar_id << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in frame-set-tool-bar: could not find frame " << id << ".\n";
      return FALSE;
    }
}

// Create the frame's status line
Bool wxcFrameCreateStatusLine (WXID id, int number)
{
  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, CLASSINFO(wxFrame));
  if (frame)
    {
      frame->CreateStatusLine (number);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in frame-create-status-line: could not find frame " << id << ".\n";
      return FALSE;
    }
}

// Set the status line text
Bool wxcFrameSetStatusText (WXID id, char *text, int i)
{
  if (!text)
    return FALSE;

  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, CLASSINFO(wxFrame));
  if (frame)
    {
      frame->SetStatusText (text, i);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in frame-set-status-text: could not find frame " << id << ".\n";
      return FALSE;
    }
}

// Set the icon
Bool wxcFrameSetIcon (WXID id, WXID iconId)
{
  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, CLASSINFO(wxFrame));
  if (frame)
    {
      wxIcon *icon = (wxIcon *) wxGetTypedObject (iconId, CLASSINFO(wxIcon));
      if (icon)
	{
	  frame->SetIcon (icon);
	  return TRUE;
	}
      else
	{
	  ExtensionError << "Error in frame-set-icon: could not find icon " << iconId << ".\n";
	  return FALSE;
	}
    }
  else
    {
      ExtensionError << "Error in frame-set-icon: could not find frame " << id << ".\n";
      return FALSE;
    }
}

// Iconimize/restore the frame
Bool wxcFrameIconize (WXID id, Bool iconize)
{
  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, CLASSINFO(wxFrame));
  if (frame)
    {
      frame->Iconize (iconize);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in frame-iconize: could not find frame " << id << ".\n";
      return FALSE;
    }
}


Bool wxcFrameSetTitle (WXID id, char *text)
{
  if (!text)
    return FALSE;

  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, CLASSINFO(wxFrame));
  if (frame)
    {
      frame->SetTitle (text);
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in frame-set-status-text: could not find frame "
	<< id << ".\n";
      return FALSE;
    }
}

char *wxcFrameGetTitle (WXID id)
{
  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, CLASSINFO(wxFrame));
  if (frame)
    {
      return frame->GetTitle ();
    }
  else
    {
      ExtensionError << "Error in frame-set-status-text: could not find frame "
	<< id << ".\n";
      return 0;
    }
}

void wxcFrameOnSize(WXID id, int w, int h)
{
  wxFrame *frame = (wxFrame *) wxGetTypedObject (id, CLASSINFO(wxFrame));
  if (frame)
    {
      frame->wxFrame::OnSize(w, h);
    }
  else
    {
      ExtensionError << "Error in frame-on-size: could not find frame "
	<< id << ".\n";
    }
}

// Creates a new item from a resource
wxItem *wxClipsPanelCreateItem(wxPanel *panel, wxItemResource *resource, wxResourceTable *table)
{
  wxItem *control = NULL;
  wxString resType(resource->GetType());
  
  if (resType == "wxButton")
      {
        if (resource->GetValue4())
        {
          // Bitmap button
          wxBitmap *bitmap = resource->GetBitmap();
          if (!bitmap)
          {
            bitmap = wxResourceCreateBitmap(resource->GetValue4(), table);
            resource->SetBitmap(bitmap);
          }
          if (bitmap)
           control = new wxExtensionButton(panel, (wxFunction)NULL, bitmap,
             resource->GetX(), resource->GetY(), resource->GetWidth(), resource->GetHeight(),
             resource->GetStyle(), resource->GetName());
        }
        else
          // Normal, text button
          control = new wxExtensionButton(panel, (wxFunction)NULL, resource->GetTitle(),
           resource->GetX(), resource->GetY(), resource->GetWidth(), resource->GetHeight(),
           resource->GetStyle(), resource->GetName());
      }
  else if (resType == "wxMessage")
      {
        if (resource->GetValue4())
        {
          // Bitmap button
          wxBitmap *bitmap = resource->GetBitmap();
          if (!bitmap)
          {
            bitmap = wxResourceCreateBitmap(resource->GetValue4(), table);
            resource->SetBitmap(bitmap);
          }
#if USE_BITMAP_MESSAGE
          if (bitmap)
           control = new wxExtensionMessage(panel, bitmap,
             resource->GetX(), resource->GetY(), // resource->GetWidth(), resource->GetHeight(),
             resource->GetStyle(), resource->GetName());
#endif
        }
        else
        {
           control = new wxExtensionMessage(panel, resource->GetTitle(),
             resource->GetX(), resource->GetY(), // resource->GetWidth(), resource->GetHeight(),
             resource->GetStyle(), resource->GetName());
        }
      }
  else if (resType == "wxText")
      {
        control = new wxExtensionText(panel, (wxFunction)NULL, resource->GetTitle(), resource->GetValue4(),
           resource->GetX(), resource->GetY(), resource->GetWidth(), resource->GetHeight(),
           resource->GetStyle(), resource->GetName());
      }
  else if (resType == "wxMultiText")
      {
        control = new wxExtensionMultiText(panel, (wxFunction)NULL, resource->GetTitle(), resource->GetValue4(),
           resource->GetX(), resource->GetY(), resource->GetWidth(), resource->GetHeight(),
           resource->GetStyle(), resource->GetName());
      }
  else if (resType == "wxCheckBox")
      {
        control = new wxExtensionCheckBox(panel, (wxFunction)NULL, resource->GetTitle(),
           resource->GetX(), resource->GetY(), resource->GetWidth(), resource->GetHeight(),
           resource->GetStyle(), resource->GetName());
        ((wxCheckBox *)control)->SetValue((Bool)resource->GetValue1());
      }
#if USE_GAUGE
  else if (resType == "wxGauge")
      {
        control = new wxExtensionGauge(panel, resource->GetTitle(), (int)resource->GetValue2(),
           resource->GetX(), resource->GetY(), resource->GetWidth(), resource->GetHeight(),
           resource->GetStyle(), resource->GetName());
        ((wxGauge *)control)->SetValue((int)resource->GetValue1());
      }
#endif
/*
#if USE_SCROLLBAR
  else if (resType == "wxScrollBar")
      {
        control = new wxExtensionScrollBar(panel, (wxFunction)NULL,
           resource->GetX(), resource->GetY(),
           resource->GetWidth(), resource->GetHeight(), 
           resource->GetStyle(), resource->GetName());
        ((wxScrollBar *)control)->SetValue((int)resource->GetValue1());
        ((wxScrollBar *)control)->SetPageLength((int)resource->GetValue2());
        ((wxScrollBar *)control)->SetObjectLength((int)resource->GetValue3());
        ((wxScrollBar *)control)->SetViewLength((int)(long)resource->GetValue5());
      }
#endif
*/
  else if (resType == "wxSlider")
      {
        control = new wxExtensionSlider(panel, (wxFunction)NULL, resource->GetTitle(), (int)resource->GetValue1(),
           (int)resource->GetValue2(), (int)resource->GetValue3(),
           resource->GetWidth(), resource->GetX(), resource->GetY(),
           resource->GetStyle(), resource->GetName());
      }
  else if (resType == "wxGroupBox")
      {
        control = new wxExtensionGroupBox(panel, resource->GetTitle(),
           resource->GetX(), resource->GetY(), resource->GetWidth(), resource->GetHeight(),
           resource->GetStyle(), resource->GetName());
      }
  else if (resType == "wxListBox")
      {
        wxStringList *stringList = resource->GetStringValues();
        char **strings = NULL;
        int noStrings = 0;
        if (stringList && (stringList->Number() > 0))
        {
          noStrings = stringList->Number();
          strings = new char *[noStrings];
          wxNode *node = stringList->First();
          int i = 0;
          while (node)
          {
            strings[i] = (char *)node->Data();
            i ++;
            node = node->Next();
          }
        }
        control = new wxExtensionListBox(panel, (wxFunction)NULL, resource->GetTitle(), (int)resource->GetValue1(),
           resource->GetX(), resource->GetY(), resource->GetWidth(), resource->GetHeight(),
           noStrings, strings, resource->GetStyle(), resource->GetName());

        if (strings)
          delete[] strings;
      }
  else if (resType == "wxChoice")
      {
        wxStringList *stringList = resource->GetStringValues();
        char **strings = NULL;
        int noStrings = 0;
        if (stringList && (stringList->Number() > 0))
        {
          noStrings = stringList->Number();
          strings = new char *[noStrings];
          wxNode *node = stringList->First();
          int i = 0;
          while (node)
          {
            strings[i] = (char *)node->Data();
            i ++;
            node = node->Next();
          }
        }
        control = new wxExtensionChoice(panel, (wxFunction)NULL, resource->GetTitle(),
           resource->GetX(), resource->GetY(), resource->GetWidth(), resource->GetHeight(),
           noStrings, strings, resource->GetStyle(), resource->GetName());

        if (strings)
          delete[] strings;
      }
  else if (resType == "wxRadioBox")
      {
        wxStringList *stringList = resource->GetStringValues();
        char **strings = NULL;
        int noStrings = 0;
        if (stringList && (stringList->Number() > 0))
        {
          noStrings = stringList->Number();
          strings = new char *[noStrings];
          wxNode *node = stringList->First();
          int i = 0;
          while (node)
          {
            strings[i] = (char *)node->Data();
            i ++;
            node = node->Next();
          }
        }
        control = new wxExtensionRadioBox(panel, (wxFunction)NULL, resource->GetTitle(),
           resource->GetX(), resource->GetY(), resource->GetWidth(), resource->GetHeight(),
           noStrings, strings, (int)resource->GetValue1(), resource->GetStyle(), resource->GetName());

        if (strings)
          delete[] strings;

      }
  if (control && resource->GetLabelFont())
    control->SetLabelFont(resource->GetLabelFont());
  if (control && resource->GetButtonFont())
    control->SetButtonFont(resource->GetButtonFont());
  return control;
}
