/*
 * wxextend.h
 * Extension language library: main header file
 *
 */

#ifndef wxextendh
#define wxextendh

#include "wx_defs.h"

#define WXEXTEND_VERSION    1.3

class ExtensionErrorClass
{
  public:
    ExtensionErrorClass(void) { }
    ~ExtensionErrorClass(void) { }

   ExtensionErrorClass& operator << (char *s);
   ExtensionErrorClass& operator << (char c);
   ExtensionErrorClass& operator << (int i);
   ExtensionErrorClass& operator << (long i);
   ExtensionErrorClass& operator << (double d);
   ExtensionErrorClass& operator << (float f);
};

// Implementation-specific extension function, to implement
// call-back methods.
typedef void (*wxExtensionFunction) (void);

// An identifier for a wxWindows object.
typedef long WXID;

// Representation of RGB colour
typedef long WXCOLOUR;

extern ExtensionErrorClass ExtensionError;

// Must be defined by user of library
extern void ExtensionErrorFunction(char *s);

// Must be called by specific language binding to implement
// callbacks. Register the most specific event handler LAST,
// i.e. register a wxWindow OnPaint handler first, then
// the wxCanvas OnPaint handler.
void wxRegisterExtensionCallback(wxClassInfo *objectType, char *eventType,
  wxExtensionFunction cFunction);

// Find a callback based on object type and event. Finds the
// most specific handler that's applicable to this event/object
// (last to be registered for this event)
wxExtensionFunction wxFindExtensionCallback(wxClassInfo *objectType, char *eventType);

// Initialize Extension library
void wxInitExtension(void);

// Cleanup Extension library
void wxCleanupExtension(void);

// Utility functions
wxObject *wxGetTypedObject(long objectId, wxClassInfo *expectedType);
void wxStoreThingInTable(long item_id, wxObject *object, wxClassInfo *type = NULL);
void wxDeleteThingInTable(long item_id);
wxObject *wxGetThingFromTable(long item_id, wxClassInfo **type = NULL);
long wxExtensionParseBitList(char *bitListString);
void wxCleanWindows(void);
Bool wxAssociateUserData(WXID id, void* dataPtr);
void* wxGetUserData(WXID id);

 /*
  * I made these public so that the names and values can be inserted
  * into the extension language's namespace... (Are there other pertinent
  * itentifiers that are missing?)
  */
struct wxExtensionBitListStruct
{
  char *word;
  long bits;
};

extern wxExtensionBitListStruct wxExtensionBitListTable[];
extern wxExtensionBitListStruct wxExtensionCursorTable[];
extern wxExtensionBitListStruct wxExtensionKeyCodeTable[];
extern int wxExtensionBitListCount;
extern int wxExtensionCursorCount;
extern int wxExtensionKeyCodeCount;

// Key code int<->string conversion
char *wxExtensionKeyCodeIntToString(long keyCode);
int wxExtensionKeyCodeStringToInt(char *keyCode);

/*
 * Command prototypes
 *
 */

char *wxcGetPlatform(void);
char *wxcGetOsVersion(void);
void wxcCleanWindows(void);

// Object
char *wxcGetObjectType(WXID id);

// Window
Bool wxcWindowDelete(WXID id);
Bool wxcWindowShow(WXID id, Bool show = TRUE);
Bool wxcWindowIsShown(WXID id);
long wxcWindowGetParent(WXID id);
int wxcWindowGetWidth(WXID id);
int wxcWindowGetHeight(WXID id);
int wxcWindowGetClientWidth(WXID id);
int wxcWindowGetClientHeight(WXID id);
int wxcWindowGetX(WXID id);
int wxcWindowGetY(WXID id);
Bool wxcWindowSetSize(WXID id, int x, int y, int w, int h);
Bool wxcWindowSetClientSize(WXID id, int w, int h);
Bool wxcWindowSetFocus(WXID id);
Bool wxcWindowSetCursor(WXID id, WXID cursorId);
Bool wxcWindowFit(WXID id);
Bool wxcWindowCentre(WXID id, int which = wxBOTH);
Bool wxcWindowCentre(WXID id, char *which);
Bool wxcWindowMakeModal(WXID id, Bool modal);
long wxcWindowGetSize(WXID id, int *w, int *h);
long wxcWindowGetClientSize(WXID id, int *w, int *h);
long wxcWindowGetPosition(WXID id, int *x, int *y);
long wxcWindowEnable(WXID id, Bool enable);
float wxcWindowGetCharWidth(WXID id);
float wxcWindowGetCharHeight(WXID id);
long wxcWindowGetTextExtent(WXID id, char* text, float* width, float* height);
char *wxcWindowGetName(WXID id);
long wxcWindowGetNextChild(WXID id, WXID child);
long wxcWindowClose(WXID id, long force = 0);
long wxcWindowSetSizeHints(WXID id, int minW=-1, int minH=-1, int maxW=-1, int maxH=-1, int incW=-1, int incH=-1);
long wxcWindowRefresh(WXID id, Bool eraseBackground=TRUE, int x = -1, int y = -1, int w = -1, int h = -1);

// Windows only
long wxcGetActiveWindow(void);

// Popup Menu Window
Bool wxcWindowPopupMenu(WXID id, WXID menuId, float x, float y);

// Panel item
Bool wxcPanelItemSetDefault(WXID id);
Bool wxcPanelItemSetLabel(WXID id, char *label);
char *wxcPanelItemGetLabel(WXID id);
WXID wxcPanelCreate(WXID parentId, int x = -1, int y = -1, int width = -1, int height = -1, long style = 0, char *name = "panel");
WXID wxcPanelCreate(WXID parentId, int x, int y, int width, int height, char *style, char *name = "panel");
WXID wxcPanelCreateFromResource(WXID parentId, char *resourceName);
Bool wxcPanelSetLabelPosition(WXID id, int position);
Bool wxcPanelSetLabelPosition(WXID id, char *position);
Bool wxcPanelSetLabelFont(WXID id, WXID fontId);
Bool wxcPanelSetButtonFont(WXID id, WXID fontId);
Bool wxcPanelNewLine(WXID id);
Bool wxcPanelOnPaint(WXID id);
Bool wxcPanelOnEvent(WXID id, WXID eventId);
long wxcPanelGetHorizontalSpacing(WXID id);
long wxcPanelGetVerticalSpacing(WXID id);
Bool wxcPanelSetHorizontalSpacing(WXID id, long spacing);
Bool wxcPanelSetVerticalSpacing(WXID id, long spacing);

// Dialog box
WXID wxcDialogBoxCreate(WXID parentId, char *title, Bool modal = FALSE,
  int x = -1, int y = -1, int width = -1, int height = -1, long style = wxCAPTION | wxSYSTEM_MENU | wxTHICK_FRAME,
  char *name = "dialogBox");
WXID wxcDialogBoxCreate(WXID parentId, char *title, Bool modal,
  int x, int y, int width, int height, char *style, char *name = "dialogBox");
WXID wxcDialogBoxCreateFromResource(WXID parentId, char *resourceName);
Bool wxcDialogBoxIsModal(WXID id);
void wxcDialogBoxSetModal(WXID id, Bool flag);

// Canvas
WXID wxcCanvasCreate(WXID parentId, int x = -1, int y = -1, int width = -1, int height = -1, long style = wxRETAINED,
  char *name = "canvas");
WXID wxcCanvasCreate(WXID parentId, int x, int y, int width, int height, char *style, char *name = "canvas");
Bool wxcCanvasSetScrollbars(WXID id, int horiz, int vert,
  int x_length, int y_length, int x_page, int y_page, int x_pos =0, int y_pos = 0);
Bool wxcCanvasScroll(WXID id, int x_pos, int y_pos);
WXID wxcCanvasGetDC(WXID id);
Bool wxcCanvasOnEvent(WXID id, WXID eventId);
Bool wxcCanvasOnChar(WXID id, WXID eventId);
Bool wxcCanvasOnScroll(WXID id, WXID eventId);
int wxcCanvasViewStartX(WXID id);
int wxcCanvasViewStartY(WXID id);
void wxcCanvasSetScrollPageX(WXID id, int page);
void wxcCanvasSetScrollPageY(WXID id, int page);
void wxcCanvasSetScrollPosX(WXID id, int pos);
void wxcCanvasSetScrollPosY(WXID id, int pos);
void wxcCanvasSetScrollRangeX(WXID id, int range);
void wxcCanvasSetScrollRangeY(WXID id, int range);
int wxcCanvasGetScrollPageX(WXID id);
int wxcCanvasGetScrollPageY(WXID id);
int wxcCanvasGetScrollPosX(WXID id);
int wxcCanvasGetScrollPosY(WXID id);
int wxcCanvasGetScrollRangeX(WXID id);
int wxcCanvasGetScrollRangeY(WXID id);
int wxcCanvasGetScrollPixelsPerUnitX(WXID id);
int wxcCanvasGetScrollPixelsPerUnitY(WXID id);

// Text window
WXID wxcTextWindowCreate(WXID parentId, int x = -1, int y = -1, int width = -1, int height = -1,
  long style = wxNATIVE_IMPL, char *name = "textWindow");
WXID wxcTextWindowCreate(WXID parentId, int x, int y, int width, int height,
  char *style, char *name = "textWindow");
Bool wxcTextWindowLoadFile(WXID id, char *filename);
Bool wxcTextWindowSaveFile(WXID id, char *filename);
Bool wxcTextWindowWrite(WXID id, long val);
Bool wxcTextWindowWrite(WXID id, char *val);
Bool wxcTextWindowWrite(WXID id, double val);
Bool wxcTextWindowModified(WXID id);
Bool wxcTextWindowClear(WXID id);
Bool wxcTextWindowDiscardEdits(WXID id);
Bool wxcTextWindowCopy(WXID id);
Bool wxcTextWindowCut(WXID id);
Bool wxcTextWindowPaste(WXID id);
char* wxcTextWindowGetContents(WXID id);
Bool wxcTextWindowReplace(WXID id, long from, long to, char *text);
Bool wxcTextWindowRemove(WXID id, long from, long to);
Bool wxcTextWindowSetEditable(WXID id, Bool editable);
Bool wxcTextWindowSetFont(WXID id, WXID fontId);
Bool wxcTextWindowSetSelection(WXID id, long start, long end);
Bool wxcTextWindowOnChar(WXID id, WXID eventId);
long wxcTextWindowGetLastPosition(WXID id);
Bool wxcTextWindowShowPosition(WXID id, long position);
long wxcTextWindowXYToPosition(WXID id, long x, long y);
Bool wxcTextWindowPositionToXY(WXID id, long position, long *x, long *y);

// New Nov. 13 1995
long wxcTextWindowPositionToLine(WXID id, long position);
long wxcTextWindowPositionToChar(WXID id, long position);
Bool wxcTextWindowSetInsertionPoint(WXID id, long position);
long wxcTextWindowGetInsertionPoint(WXID id);
long wxcTextWindowGetLineLength(WXID id, long lineNo);
char *wxcTextWindowGetLineText(WXID id, long lineNo);
long wxcTextWindowGetNumberOfLines(WXID id);

// Generic device context functionality
Bool wxcDCOk(WXID id);
Bool wxcDCStartDoc(WXID id, char *message = NULL);
Bool wxcDCEndDoc(WXID id);
Bool wxcDCStartPage(WXID id);
Bool wxcDCEndPage(WXID id);
Bool wxcDCSetPen(WXID id, long penId);
Bool wxcDCSetBackground(WXID id, WXID brushId);
Bool wxcDCSetBrush(WXID id, long brushId);
Bool wxcDCSetMapMode(WXID id, int mode);
Bool wxcDCSetMapMode(WXID id, char *mode);
Bool wxcDCSetFont(WXID id, long fontId);
Bool wxcDCSetTextForeground(WXID id, char *colour);
Bool wxcDCSetTextBackground(WXID id, char *colour);
Bool wxcDCSetLogicalFunction(WXID id, int op);
Bool wxcDCSetBackgroundMode(WXID id, int mode);
Bool wxcDCSetBackgroundMode(WXID id, char *mode);
// String version
Bool wxcDCSetLogicalFunction(WXID id, char *op);
Bool wxcDCSetColourMap(WXID id, WXID cmapId);
Bool wxcDCBlit(WXID id, double xdest, double ydest, double width, double height, WXID sourceId,
  double xsrc, double ysrc, int op);
// String version
Bool wxcDCBlit(WXID id, double xdest, double ydest, double width, double height, long sourceId,
  double xsrc, double ysrc, char *op);
Bool wxcDCDrawIcon(WXID id, WXID icon_id, float x, float y);
Bool wxcDCDrawLine(WXID id, double x1, double y1, double x2, double y2);
Bool wxcDCDrawPoint(WXID id, double x1, double y1);
Bool wxcDCDrawRectangle(WXID id, double x1, double y1, double x2, double y2);
Bool wxcDCDrawRoundedRectangle(WXID id, double x1, double y1, double x2, double y2,
  double radius = 20);
Bool wxcDCDrawEllipse(WXID id, double x, double y, double width, double height);
Bool wxcDCDrawText(WXID id, char *text, double x1, double y1);
Bool wxcDCSetClippingRegion(WXID id, double x1, double y1, double x2, double y2);
Bool wxcDCDestroyClippingRegion(WXID id);
Bool wxcDCClear(WXID id);
double wxcDCGetWidth(WXID id);
double wxcDCGetHeight(WXID id);
double wxcDCGetMinX(WXID id);
double wxcDCGetMinY(WXID id);
double wxcDCGetMaxX(WXID id);
double wxcDCGetMaxY(WXID id);
Bool wxcDCSetUserScale(WXID id, double x, double y);
Bool wxcDCSetDeviceOrigin(WXID id, double x, double y);
Bool wxcDCSetLogicalOrigin(WXID id, double x, double y);
Bool wxcDCDelete(WXID id);
long wxcDCGetSize(WXID id, float *w, float *h);
long wxcDCDrawArc(long id, float x1, float y1, float x2, float y2,
             float xc, float yc);
long wxcDCGetTextExtent(long id, char* text, float* width, float* height);
long wxcDCBeginDrawing(WXID id);
long wxcDCEndDrawing(WXID id);

// Colours: colour is stored inside a long integer, not a class.
WXCOLOUR wxcColourCreate(int red, int green, int blue);
WXCOLOUR wxcColourCreate(char *colour);
int wxcColourRed(WXCOLOUR col);
int wxcColourGreen(WXCOLOUR col);
int wxcColourBlue(WXCOLOUR col);

// PostScript DC
WXID wxcPostScriptDCCreate(char *file = NULL, Bool interactive = TRUE, WXID parentId = 0);

#ifdef wx_msw
// Windows Printer DC
WXID wxcPrinterDCCreate(char *driver = NULL, char *device = NULL, char *file = NULL, Bool interactive = TRUE);

// Windows metafile DC
WXID wxcMetaFileDCCreate(char *filename = NULL);
WXID wxcMetaFileDCClose(WXID id);

// Windows metafile
Bool wxcMetaFileDelete(WXID id);
Bool wxcMetaFileSetClipboard(WXID id, double width = 0.0, double height = 0.0);
Bool wxcMakeMetaFilePlaceable(char *filename, int minX, int minY, int maxX, int maxY, double scale = 1.0);
#endif

// Memory DC
WXID wxcMemoryDCCreate(void);
Bool wxcMemoryDCSelectObject(WXID id, long bitmapId);

// Application object: returns a pointer to the current wxApp
WXID wxcAppCreate(void);
void wxcAppSetShowFrameOnInit(WXID id, Bool flag);
Bool wxcAppGetShowFrameOnInit(WXID id);

// Pen
WXID wxcPenCreate(char *colour, int width, int style = wxSOLID);
WXID wxcPenCreate(WXCOLOUR colour, int width, int style = wxSOLID);
WXID wxcPenCreate(char *colour, int width, char *style);
WXID wxcPenCreate(WXCOLOUR colour, int width, char *style);
Bool wxcPenDelete(WXID id);

// Brush
WXID wxcBrushCreate(char *colour, int style = wxSOLID);
WXID wxcBrushCreate(WXCOLOUR colour, int style = wxSOLID);
WXID wxcBrushCreate(char *colour, char *style);
WXID wxcBrushCreate(WXCOLOUR colour, char *style);
Bool wxcBrushDelete(WXID id);

// Font
WXID wxcFontCreate(int pointSize, int family, int style, int weight,
  Bool underlined = 0);
WXID wxcFontCreate(int pointSize, char *family, char *style, char *weight,
  Bool underlined = 0);
Bool wxcFontDelete(WXID id);

// Bitmap
WXID wxcBitmapCreate(int width, int height, int depth = -1);
Bool wxcBitmapDelete(WXID id);
WXID wxcBitmapLoadFromFile(char *file, char *bitmapType);
// Integer version
WXID wxcBitmapLoadFromFile(char *file, long bitmapType);

int wxcBitmapGetWidth(WXID id);
int wxcBitmapGetHeight(WXID id);
int wxcBitmapGetDepth(WXID id);
WXID wxcBitmapGetColourMap(WXID id);

// Cursor
WXID wxcCursorCreate(char *cursorType);
WXID wxcCursorCreate(int cursorType);
WXID wxcCursorLoadFromFile(char *file, char *bitmapType = NULL, int x = 0, int y = 0);
Bool wxcCursorDelete(WXID id);

// Icon
WXID wxcIconCreate(char *file);
WXID wxcIconLoadFromFile(char *file, char *bitmapType = NULL);
Bool wxcIconDelete(WXID id);
int wxcIconGetWidth(WXID id);
int wxcIconGetHeight(WXID id);

// Event
long wxcEventGetEventType(WXID id);
long wxcEventGetEventClass(WXID id);

// Key event
Bool wxcKeyEventControlDown(WXID id);
Bool wxcKeyEventShiftDown(WXID id);
Bool wxcKeyEventAltDown(WXID id);
long wxcKeyEventKeyCode(WXID id);
char *wxcKeyEventKeyCodeString(WXID id);
Bool wxcKeyEventPosition(WXID id, double *x, double *y);
 
// Command event
char* wxcCommandEventGetClientData(WXID id);
long wxcCommandEventGetSelection(WXID id);
char* wxcCommandEventGetString(WXID id);
Bool wxcCommandEventChecked(WXID id);
Bool wxcCommandEventIsSelection(WXID id);

// Mouse event
Bool wxcMouseEventIsButton(WXID id);
Bool wxcMouseEventButtonDown(WXID id);
Bool wxcMouseEventControlDown(WXID id);
Bool wxcMouseEventShiftDown(WXID id);
Bool wxcMouseEventButton(WXID id, int butt);
Bool wxcMouseEventLeftDown(WXID id);
Bool wxcMouseEventMiddleDown(WXID id);
Bool wxcMouseEventRightDown(WXID id);
Bool wxcMouseEventLeftUp(WXID id);
Bool wxcMouseEventMiddleUp(WXID id);
Bool wxcMouseEventRightUp(WXID id);
Bool wxcMouseEventDragging(WXID id);
double wxcMouseEventPositionX(WXID id);
double wxcMouseEventPositionY(WXID id);
long wxcMouseEventPosition(long id, double *x, double *y);

// Button
WXID wxcButtonCreate(WXID parentId, char *label,
  int x = -1, int y = -1, int width = -1, int height = -1, long style = 0, char *name = "button");
WXID wxcButtonCreate(WXID parentId, char *label,
  int x, int y, int width, int height, char *style, char *name = "button");
WXID wxcButtonCreateFromBitmap(WXID parentId, WXID bitmapId,
  int x = -1, int y = -1, int width = -1, int height = -1, long style = 0, char *name = "button");
WXID wxcButtonCreateFromBitmap(WXID parentId, WXID bitmapId,
  int x, int y, int width, int height, char *style, char *name = "button");
Bool wxcButtonSetLabel(WXID id, WXID bitmapId);
Bool wxcButtonSetLabel(WXID id, char *label);

// Text control
WXID wxcTextCreate(WXID parentId, char *label, char *value = "",
  int x = -1, int y = -1, int width = -1, int height = -1, long style = 0, char *name = "text");
WXID wxcTextCreate(WXID parentId, char *label, char *value,
  int x, int y, int width, int height, char *style, char *name = "text");
Bool wxcTextSetValue(WXID id, char *val);
char *wxcTextGetValue(WXID id);

// Multi-line text control
WXID wxcMultiTextCreate(WXID parentId, char *label, char *value = "",
  int x = -1, int y = -1, int width = -1, int height = -1, long style = 0, char *name = "multiText");
WXID wxcMultiTextCreate(WXID parentId, char *label, char *value,
  int x, int y, int width, int height, char *style, char *name = "multiText");
Bool wxcMultiTextSetValue(WXID id, char *val);
char *wxcMultiTextGetValue(WXID id);

// Finer control for multitext: MS Windows only
#ifdef wx_msw
Bool wxcMultiTextWrite(WXID id, char *val);
Bool wxcMultiTextCopy(WXID id);
Bool wxcMultiTextCut(WXID id);
Bool wxcMultiTextPaste(WXID id);
Bool wxcMultiTextReplace(WXID id, long from, long to, char *text);
Bool wxcMultiTextRemove(WXID id, long from, long to);
Bool wxcMultiTextSetSelection(WXID id, long start, long end);
long wxcMultiTextGetLastPosition(WXID id);
Bool wxcMultiTextShowPosition(WXID id, long position);
long wxcMultiTextXYToPosition(WXID id, long x, long y);
Bool wxcMultiTextPositionToXY(WXID id, long position, long *x, long *y);
long wxcMultiTextPositionToLine(WXID id, long position);
long wxcMultiTextPositionToChar(WXID id, long position);
Bool wxcMultiTextSetInsertionPoint(WXID id, long position);
long wxcMultiTextGetInsertionPoint(WXID id);
long wxcMultiTextGetLineLength(WXID id, long lineNo);
char *wxcMultiTextGetLineText(WXID id, long lineNo);
long wxcMultiTextGetNumberOfLines(WXID id);
#endif

// Checkbox
WXID wxcCheckBoxCreate(WXID parentId, char *label,
  int x = -1, int y = -1, int width = -1, int height = -1, long style = 0, char *name = "checkBox");
WXID wxcCheckBoxCreate(WXID parentId, char *label,
  int x, int y, int width, int height, char *style, char *name = "checkBox");
Bool wxcCheckBoxSetValue(WXID id, Bool val);
int wxcCheckBoxGetValue(WXID id);

// Slider
WXID wxcSliderCreate(WXID parentId, char *label,
  int value, int min_value, int max_value, int width, int x = -1, int y = -1,
  long style = 0, char *name = "slider");
WXID wxcSliderCreate(WXID parentId, char *label,
  int value, int min_value, int max_value, int width, int x, int y,
  char *style, char *name = "slider");
Bool wxcSliderSetValue(WXID id, int val);
int wxcSliderGetValue(WXID id);

// Message (static text)
WXID wxcMessageCreate(WXID parentId, char *label, int x = -1, int y = -1,
  long style = 0, char *name = "message");
WXID wxcMessageCreate(WXID parentId, char *label, int x, int y,
  char *style, char *name = "message");
WXID wxcMessageCreateFromBitmap(WXID parentId, WXID bitmapId, int x = -1, int y = -1,
  long style = 0, char *name = "message");
WXID wxcMessageCreateFromBitmap(WXID parentId, WXID bitmapId, int x, int y,
  char *style, char *name = "message");

// Listbox
WXID wxcListBoxCreate(WXID parentId, char *label, Bool multiple = FALSE,
  int x = -1, int y = -1, int width = -1, int height = -1, long style = 0, char *name = "listbox");
WXID wxcListBoxCreate(WXID parentId, char *label, Bool multiple,
  int x, int y, int width, int height, char *style, char *name = "listbox");
Bool wxcListBoxAppend(WXID id, char *item_string, char *client_data = NULL);
int wxcListBoxFindString(WXID id, char *item_string);
Bool wxcListBoxClear(WXID id);
Bool wxcListBoxSetSelection(WXID id, int sel);
Bool wxcListBoxDeselect(WXID id, int sel);
int wxcListBoxGetSelection(WXID id);
char *wxcListBoxGetStringSelection(WXID id);
Bool wxcListBoxSetStringSelection(WXID id, char *sel);
int wxcListBoxNumber(WXID id);
Bool wxcListBoxDelete(WXID id, int n);
char *wxcListBoxGetClientData(WXID id, int sel);
char *wxcListBoxGetString(WXID id, int sel);
int wxcListBoxGetFirstSelection(WXID id);
int wxcListBoxGetNextSelection(void);
long wxcListBoxSetFirstItem(WXID id, long sel);

// Choice (non-editable combobox)
// Implements the choice creation independent of ClipsExpr or CLIPS
WXID wxcChoiceCreate(WXID parentId, char *label, int x, int y,
  int width, int height, int n, char **choices, long style = 0, char *name = "choice");
WXID wxcChoiceCreate(WXID parentId, char *label, int x, int y,
  int width, int height, int n, char **choices, char *style, char *name = "choice");
Bool wxcChoiceAppend(WXID id, char *item_string);
int wxcChoiceFindString(WXID id, char *item_string);
Bool wxcChoiceClear(WXID id);
Bool wxcChoiceSetSelection(WXID id, int sel);
int wxcChoiceGetSelection(WXID id);
char *wxcChoiceGetStringSelection(WXID id);
Bool wxcChoiceSetStringSelection(WXID id, char *sel);
char *wxcChoiceGetString(WXID id, int sel);

// RadioBox
WXID wxcRadioBoxCreate(WXID parentId, char *label, int x, int y,
                       int width, int height, int n, char **choices,
                       int majorDim, long style, char *name = "radioBox");
WXID wxcRadioBoxCreate(WXID parentId, char *label, int x, int y,
                       int width, int height, int n, char **choices,
                       int majorDim, char *style, char *name = "radioBox");
long wxcRadioBoxSetSelection(WXID id, long sel);
long wxcRadioBoxGetSelection(WXID id);
 
// Gauge
#if USE_GAUGE
WXID wxcGaugeCreate(WXID parentId, char *label, int range,
  int x = -1, int y = -1, int width = -1, int height = -1, long style = 0, char *name = "gauge");
WXID wxcGaugeCreate(WXID parentId, char *label, int range,
  int x, int y, int width, int height, char *style, char *name = "gauge");
Bool wxcGaugeSetValue(WXID parentId, int val);
Bool wxcGaugeSetBezelFace(WXID parentId, int width);
Bool wxcGaugeSetShadowWidth(WXID parentId, int width);
Bool wxcGaugeSetRange(WXID id, int value);
#endif

// Group box
WXID wxcGroupBoxCreate(WXID parentId, char *label,
  int x, int y, int width, int height, long style = 0, char *name = "groupBox");
WXID wxcGroupBoxCreate(WXID parentId, char *label,
  int x, int y, int width, int height, char *style, char *name = "groupBox");

// Menu
WXID wxcMenuCreate(char *title = NULL, wxFunction func = 0);
Bool wxcMenuAppend(WXID id, int item_id, char *menu_string, WXID submenu_id = -1, char *help_string = NULL, Bool checkable = FALSE);
Bool wxcMenuAppend(WXID id, int item_id, char *menu_string, char *help_string, Bool checkable);
Bool wxcMenuAppendSeparator(WXID id);
Bool wxcMenuBreak(WXID id);
Bool wxcMenuEnable(WXID id, int item_id, Bool enabled);
Bool wxcMenuCheck(WXID id, int item_id, Bool checked);
long wxcMenuSetLabel(WXID id, long item_id, char *label);

// Menubar
WXID wxcMenuBarCreate(void);
Bool wxcMenuBarAppend(WXID id, WXID menu_id, char *menu_string);
Bool wxcMenuBarEnable(WXID id, int item_id, Bool enabled);
Bool wxcMenuBarCheck(WXID id, int item_id, Bool checked);
Bool wxcMenuBarChecked(WXID id, int item_id);

// Frame
WXID wxcFrameCreate(WXID parentId, char *title, int x = -1, int y = -1,
  int width = -1, int height = -1, long style = wxDEFAULT_FRAME, char *name = "frame");
WXID wxcFrameCreate(WXID parentId, char *title, int x, int y,
  int width, int height, char *style, char *name = "frame");
Bool wxcFrameSetMenuBar(WXID id, WXID menubar_id);
Bool wxcFrameSetToolBar(WXID id, WXID toolbar_id);
Bool wxcFrameCreateStatusLine(WXID id, int number=1);
Bool wxcFrameSetStatusText(WXID id, char *text, int i = 0);
Bool wxcFrameSetIcon(WXID id, WXID iconId);
Bool wxcFrameIconize(WXID id, Bool iconize = TRUE);
Bool wxcFrameSetTitle(WXID id, char *text);
char* wxcFrameGetTitle(WXID id);
void wxcFrameOnSize(WXID id, int w, int h);

// Miscellaneous
char *wxcGetObjectType(WXID id);
char *wxcGetTextFromUser(char *message = "Enter a string",
                         char *caption = "Enter text",
                         char *default_string = "",
                         Bool centre = TRUE, WXID parentId = 0);
double wxcStringToFloat(char *str);
long wxcStringToLong(char *str);
char *wxcFloatToString(double l);
char *wxcLongToString(long l);
char *wxcStringToSymbol(char *s);
char *wxcSymbolToString(char *s);
char *wxcMessageBox(char *message, char *flags = "OK", Bool centre = TRUE, WXID parentId = 0, char *title = "Message");
char *wxcFileSelector(char *message = "Select a file",
  char *def_path = NULL, char *def_file = NULL, char *def_ext = NULL,
  char *def_wildcard = "*.*", long flags=0, WXID parentId = 0);
// Version with string flags argument
char *wxcFileSelector(char *message,
  char *def_path, char *def_file, char *def_ext, char *def_wildcard, char *flags, WXID parentId);
Bool wxcYield(void);
Bool wxcSleep(long nSecs);

// win.ini/.rc file resources
Bool wxcWriteResource(char *section, char *entry, char *value, char *filename = NULL);
char *wxcGetResource(char *section, char *entry, char *filename = NULL);

// wxWindows .wxr resource system (not to be confused with above resources)
Bool wxcLoadResourceFile(char *filename);
void wxcClearResources(void);

Bool wxcDebugMsg(char *msg);

// Timer functions
// Bool wxcTimerInit(wxClipsFunction func);
WXID wxcTimerCreate(void);
Bool wxcTimerStart(WXID id, long ms, Bool one_shot = FALSE);
Bool wxcTimerStop(WXID id);
Bool wxcTimerDelete(WXID id);

long wxcGetElapsedTime(Bool resetTimer = TRUE);
void wxcStartTimer(void);
char *wxcNow(void);

void wxcBeginBusyCursor(void);
void wxcEndBusyCursor(void);

// Toolbar functions
WXID wxcToolbarCreate(long parent, int x, int y, int w, int h, long style = 0,
   int orientation = wxVERTICAL, int rowsOrColumns = 1, Bool createButtons = 1, char *name = "toolbar");
WXID wxcToolbarCreate(WXID parent_id, int x, int y, int w, int h, char *styleString,
  char *orientation, int rowsOrColumns, Bool createButtons, char *name = "toolbar");
Bool wxcToolbarAddTool(WXID id, int index, WXID bitmapId1, WXID bitmapId2 = 0, Bool toggle = FALSE,
  double x = -1.0, double y = -1.0, long clientData = 0, char *helpString1 = NULL, char *helpString2 = NULL);
Bool wxcToolbarAddSeparator(WXID id);
Bool wxcToolbarCreateTools(WXID id);
Bool wxcToolbarClearTools(WXID id);
long wxcToolbarGetToolClientData(WXID id, int index);
char *wxcToolbarGetToolShortHelp(WXID id, int index);
char *wxcToolbarGetToolLongHelp(WXID id, int index);
Bool wxcToolbarSetToolShortHelp(WXID id, int index, char *helpString);
Bool wxcToolbarSetToolLongHelp(WXID id, int index, char *helpString);
double wxcToolbarGetMaxWidth(WXID id);
double wxcToolbarGetMaxHeight(WXID id);
Bool wxcToolbarGetToolState(WXID id, int index);
Bool wxcToolbarGetToolEnabled(WXID id, int index);
Bool wxcToolbarLayout(WXID id);
Bool wxcToolbarOnPaint(WXID id);
Bool wxcToolbarSetMargins(WXID id, double x, double y);
Bool wxcToolbarSetDefaultSize(WXID id, int x, int y);
Bool wxcToolbarEnableTool(WXID id, int tool, Bool enable);
Bool wxcToolbarToggleTool(WXID id, int tool, Bool toggle);

// DDE functions
WXID wxcClientCreate(void);
WXID wxcClientMakeConnection(WXID id, char *host, char *service, char *topic);
WXID wxcServerCreate(char *service);
Bool wxcConnectionAdvise(WXID id, char *item, char *data);
Bool wxcConnectionExecute(WXID id, char *data);
Bool wxcConnectionDisconnect(WXID id);
Bool wxcConnectionPoke(WXID id, char *item, char *data);
char *wxcConnectionRequest(WXID id, char *item);
Bool wxcConnectionStartAdvise(WXID id, char *item);
Bool wxcConnectionStopAdvise(WXID id, char *item);

// Help-accessing functions
WXID wxcHelpCreate(Bool native = TRUE);
Bool wxcHelpDelete(WXID id);
Bool wxcHelpLoadFile(WXID id, char *file);
Bool wxcHelpDisplayContents(WXID id);
Bool wxcHelpDisplaySection(WXID id, long sectionId);
Bool wxcHelpDisplayBlock(WXID id, long blockId);
Bool wxcHelpKeywordSearch(WXID id, char *keyword);
Bool wxcHelpQuit(WXID id);

// ODBC functions
#if defined(wx_msw) && USE_ODBC && WXODBC
#include "wx_db.h"

WXID wxcDatabaseCreate(void);
Bool wxcDatabaseOpen(WXID id, char *source, Bool exclusive = FALSE, Bool readOnly = TRUE,
   char *username = "ODBC", char *password = "");
Bool wxcDatabaseClose(WXID id);
Bool wxcDatabaseIsOpen(WXID id);
long wxcDatabaseGetErrorCode(WXID id);
char *wxcDatabaseGetErrorCodeString(WXID id);
char *wxcDatabaseGetDatabaseName(WXID id);
Bool wxcDatabaseErrorOccurred(WXID id);
char *wxcDatabaseGetErrorMessage(WXID id);
long wxcDatabaseGetErrorNumber(WXID id);
Bool wxcDatabaseDelete(WXID id);

WXID wxcRecordSetCreate(WXID dbId, int openType = wxOPEN_TYPE_DYNASET, int options = wxOPTION_DEFAULT);
WXID wxcRecordSetCreate(WXID dbId, char *openType = "wxOPEN_TYPE_DYNASET", char *options = "wxOPTION_DEFAULT");
Bool wxcRecordSetDelete(WXID rsId);
Bool wxcRecordSetQuery(WXID rsId, char *cols, char *table = NULL, char *filter = NULL);
int wxcRecordSetGetNumberFields(WXID rsId);
int wxcRecordSetGetNumberParams(WXID rsId);
long wxcRecordSetGetNumberRecords(WXID rsId);
long wxcRecordSetGetNumberCols(WXID rsId);
char *wxcRecordSetGetFilter(WXID rsId);
char *wxcRecordSetGetSortString(WXID rsId);
WXID wxcRecordSetGetDatabase(WXID rsId);
long wxcRecordSetGetErrorCode(WXID rsId);
char *wxcRecordSetGetErrorCodeString(WXID rsId);
Bool wxcRecordSetGetResultSet(WXID rsId);
Bool wxcRecordSetExecuteSql(WXID rsId, char *sql);
Bool wxcRecordSetGetTables(WXID rsId);
Bool wxcRecordSetGetColumns(WXID rsId, char *table = NULL);
Bool wxcRecordSetGetPrimaryKeys(WXID rsId, char *table = NULL);
Bool wxcRecordSetGetForeignKeys(WXID rsId, char *table = NULL, char *fName
  = NULL);
char *wxcRecordSetGetTableName(WXID rsId);
Bool wxcRecordSetSetTableName(WXID rsId, char *table);
Bool wxcRecordSetIsOpen(WXID rsId);
Bool wxcRecordSetIsBOF(WXID rsId);
Bool wxcRecordSetIsEOF(WXID rsId);
Bool wxcRecordSetIsDeleted(WXID rsId);
long wxcRecordSetGetIntData(WXID rsId, int colPos);
long wxcRecordSetGetIntData(WXID rsId, char *col);
double wxcRecordSetGetFloatData(WXID rsId, int colPos);
double wxcRecordSetGetFloatData(WXID rsId, char *col);
char *wxcRecordSetGetCharData(WXID rsId, int colPos);
char *wxcRecordSetGetCharData(WXID rsId, char *col);
char *wxcRecordSetGetColName(WXID rsId, int pos);
long wxcRecordSetGetColType(WXID rsId, int pos);
long wxcRecordSetGetColType(WXID rsId, char *col);
char *wxcRecordSetGetColTypeString(WXID rsId, int pos);
char *wxcRecordSetGetColTypeString(WXID rsId, char *col);
Bool wxcRecordSetMove(WXID rsId, long rows);
Bool wxcRecordSetMoveFirst(WXID rsId);
Bool wxcRecordSetMoveLast(WXID rsId);
Bool wxcRecordSetMoveNext(WXID rsId);
Bool wxcRecordSetMovePrev(WXID rsId);
Bool wxcRecordSetGoto(WXID rsId, long row);
Bool wxcRecordSetGetDataSources(WXID rsId);
Bool wxcRecordSetIsFieldDirty(WXID rsId, int col);
Bool wxcRecordSetIsFieldDirty(WXID rsId, char *col);
Bool wxcRecordSetIsFieldNull(WXID rsId, int col);
Bool wxcRecordSetIsFieldNull(WXID rsId, char *col);
Bool wxcRecordSetIsColNullable(WXID rsId, int col);
Bool wxcRecordSetIsColNullable(WXID rsId, char *col);
Bool wxcRecordSetSetFieldDirty(WXID rsId, int col, Bool dirty = TRUE);
Bool wxcRecordSetSetFieldDirty(WXID rsId, char *col, Bool dirty = TRUE);
#endif

#ifdef wx_msw
// MCI (multimedia command interface) string function. Returns
// NULL if no error, otherwise a text description of the error.
char *wxcMCISendString(char *command);

WXID wxcHWNDFind(char *title);
Bool wxcHWNDIconize(WXID hWnd, Bool iconize = TRUE);
Bool wxcHWNDShow(WXID hWnd, Bool show = TRUE);
Bool wxcHWNDMove(WXID hWnd, int x, int y, int w, int h, Bool repaint = TRUE);
Bool wxcHWNDQuit(WXID hWnd);
Bool wxcHWNDRefresh(WXID hWnd, Bool eraseBackground = TRUE);
Bool wxcHWNDSendMessage(WXID hWnd, long msg, long wParam, long lParam);

#endif

void wxcBell(void);
Bool wxcChdir(char *s);
Bool wxcMkdir(char *s);
Bool wxcRmdir(char *s);
Bool wxcFileExists(char *f);
Bool wxcDirExists(char *f);
Bool wxcCopyFile(char *f1, char *f2);

// Finding windows
WXID wxcFindWindowByName(char *title, WXID parent = 0);
WXID wxcFindWindowByLabel(char *name, WXID parent = 0);

// Date functions
WXID wxcDateCreateString(char *s);
WXID wxcDateCreateJulian(long j);
WXID wxcDateCreate(void);
WXID wxcDateCreate(const int m, const int d, const int y);
Bool wxcDateDelete(WXID dateId);
WXID wxcDateAddDays(WXID dateId, long days);
WXID wxcDateSubtractDays(WXID dateId, long days);
long wxcDateSubtract(WXID dateId1, WXID dateId2);
Bool wxcDateAddSelf(WXID dateId, long days);
Bool wxcDateSubtractSelf(WXID dateId, long days);
Bool wxcDateLE(WXID dateId1, WXID dateId2);
Bool wxcDateLEQ(WXID dateId1, WXID dateId2);
Bool wxcDateGE(WXID dateId1, WXID dateId2);
Bool wxcDateGEQ(WXID dateId1, WXID dateId2);
Bool wxcDateEQ(WXID dateId1, WXID dateId2);
Bool wxcDateNEQ(WXID dateId1, WXID dateId2);
char *wxcDateFormat(WXID dateId, const int type = -1);
Bool wxcDateSetFormat(WXID dateId, const int type);
Bool wxcDateSetFormat(WXID dateId, const char *type);
Bool wxcDateSetOption(WXID dateId, const int option, Bool enable = TRUE);
Bool wxcDateSetOption(WXID dateId, const char *option, Bool enable = TRUE);
long wxcDateGetJulianDate(WXID dateId);
int wxcDateGetDayOfYear(WXID dateId);
Bool wxcDateIsLeapYear(WXID dateId);
Bool wxcDateSetCurrentDate(WXID dateId);
Bool wxcDateSetJulian(WXID dateId, long julian);
Bool wxcDateSetDate(WXID dateId, int m, int d, int y);
Bool wxcDateAddWeeks(WXID dateId, int n);
Bool wxcDateAddMonths(WXID dateId, int n);
Bool wxcDateAddYears(WXID dateId, int n);
int wxcDateGetDay(WXID dateId);
int wxcDateGetDaysInMonth(WXID dateId);
int wxcDateGetFirstDayOfMonth(WXID dateId);
int wxcDateGetDayOfWeek(WXID dateId);
int wxcDateGetWeekOfMonth(WXID dateId);
int wxcDateGetWeekOfYear(WXID dateId);
char *wxcDateGetDayOfWeekName(WXID dateId);
char *wxcDateGetMonthName(WXID dateId);
int wxcDateGetMonth(WXID dateId);
int wxcDateGetYear(WXID dateId);
WXID wxcDateGetMonthStart(WXID dateId);
WXID wxcDateGetMonthEnd(WXID dateId);
WXID wxcDateGetYearStart(WXID dateId);
WXID wxcDateGetYearEnd(WXID dateId);

#if defined(wx_msw) && USE_WXGRID
#define wxTYPE_GRID            120

long wxcGridCreate (long parentId, int x = -1, int y = -1, int width = -1, int height = -1, long style = 0, char *name = "grid");
// Grid creation (string version)
long wxcGridCreate (long parentId, int x, int y, int width, int height, char *styleString, char *name = "grid");
Bool wxcGridCreateGrid (WXID id, int rows, int cols, int defaultWidth, int defaultHeight);
void wxcGridClearGrid (WXID id);
Bool wxcGridInsertCols (WXID id, int pos, int n = 1, Bool updateLabels = TRUE);
Bool wxcGridInsertRows (WXID id, int pos, int n = 1, Bool updateLabels = TRUE);
Bool wxcGridAppendCols (WXID id, int n=1, Bool updateLabels = TRUE);
Bool wxcGridAppendRows (WXID id, int n=1, Bool updateLabels = TRUE);
Bool wxcGridDeleteCols (WXID id, int pos, int n=1, Bool updateLabels = TRUE);
Bool wxcGridDeleteRows (WXID id, int pos, int n=1, Bool updateLabels = TRUE);
void wxcGridSetCellValue (WXID id, char *value, int row, int col);
void wxcGridSetCellBitmap (WXID id, WXID bitmapId, int row, int col);
char *wxcGridGetCellValue (WXID id, int row, int col);
WXID wxcGridGetCellBitmap (WXID id, int row, int col);
void wxcGridSetCellAlignment (WXID id, int flag, int row=-1, int col=-1);
void wxcGridSetCellAlignment (WXID id, char *flag, int row=-1, int col=-1);
int wxcGridGetCellAlignment (WXID id, int row=-1, int col=-1);
char *wxcGridGetCellAlignmentString (WXID id, int row=-1, int col=-1);
void wxcGridSetCellTextColour (WXID id, WXCOLOUR colour, int row=-1, int col=-1);
WXCOLOUR wxcGridGetCellTextColour (WXID id, int row=-1, int col=-1);
void wxcGridSetCellBackgroundColour (WXID id, WXCOLOUR colour, int row=-1, int col=-1);
WXCOLOUR wxcGridGetCellBackgroundColour (WXID id, int row=-1, int col=-1);
void wxcGridSetCellTextFont (WXID id, WXID fontId, int row=-1, int col=-1);
void wxcGridSetColumnWidth (WXID id, int col, int width);
int wxcGridGetColumnWidth (WXID id, int col);
void wxcGridSetRowHeight (WXID id, int row, int height);
int wxcGridGetRowHeight (WXID id, int row);
void wxcGridSetLabelSize (WXID id, int orientation, int sz);
void wxcGridSetLabelSize (WXID id, char *orientation, int sz);
int wxcGridGetLabelSize (WXID id, int orientation);
int wxcGridGetLabelSize (WXID id, char *orientation);
void wxcGridSetLabelAlignment (WXID id, int orientation, int alignment);
int wxcGridGetLabelAlignment (WXID id, int orientation);
void wxcGridSetLabelAlignment (WXID id, char *orientation, char *alignment);
char *wxcGridGetLabelAlignment (WXID id, char *orientation);
void wxcGridSetLabelValue (WXID id, int orientation, char *value, int pos);
void wxcGridSetLabelValue (WXID id, char *orientation, char *value, int pos);
char *wxcGridGetLabelValue (WXID id, int orientation, int pos);
char *wxcGridGetLabelValue (WXID id, char *orientation, int pos);
void wxcGridSetLabelTextColour (WXID id, WXCOLOUR colour);
WXCOLOUR wxcGridGetLabelTextColour (WXID id);
void wxcGridSetLabelBackgroundColour (WXID id, WXCOLOUR colour);
WXCOLOUR wxcGridGetLabelBackgroundColour (WXID id);
void wxcGridSetLabelTextFont (WXID id, WXID fontId);
int wxcGridGetCursorRow (WXID id);
int wxcGridGetCursorColumn (WXID id);
void wxcGridSetGridCursor (WXID id, int row, int col);
int wxcGridGetRows (WXID id);
int wxcGridGetCols (WXID id);
int wxcGridGetScrollPosX (WXID id);
int wxcGridGetScrollPosY (WXID id);
void wxcGridSetEditable (WXID id, Bool editable);
Bool wxcGridGetEditable (WXID id);
void wxcGridSetDividerPen (WXID id, WXID penId);
void wxcGridAdjustScrollbars (WXID id);
void wxcGridUpdateDimensions (WXID id);
void wxcGridOnActivate (WXID id, Bool active);
void wxcGridOnPaint (WXID id);
void wxcGridOnSize (WXID id, int w, int h);
long wxcGridGetTextItem(WXID id);
long wxcGridBeginBatch(WXID id);
long wxcGridEndBatch(WXID id);
long wxcGridGetBatchCount(WXID id);
#endif

// This should be ok for many objects, so in theory we can
// remove several wxcDelete... functions.
// EXCEPT this won't work for classes that HAVEN'T been
// derived and destructor overridden to delete the object
// from the identifier table.
Bool wxcObjectDelete(WXID id);

#endif // wxextendh
